@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.19 2003/06/11 11:42:31 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.20 2004/03/10 23:49:13 abe Exp $";
 #endif
 
 
@@ -1155,7 +1155,7 @@ readlino(ga, li)
 /*
  * Read a "standard" inode.
  */
-	if (readinode((KA_T)ga->gn_data, (char *)&i))
+	if (readinode((KA_T)ga->gn_data, &i))
 	    return(1);
 	li->dev = i.i_dev;
 	li->nlink = i.i_nlink;
