@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.23 2003/03/21 17:39:46 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.24 2004/07/06 19:16:20 abe Exp $";
 #endif
 
 
@@ -57,6 +57,14 @@ _PROTOTYPE(static void getsoinfo,(void));
 _PROTOTYPE(static void process_text,(pid_t pid));
 #endif	/* AIXA<2 */
 
+#if	defined(SIGDANGER)
+# if	defined(HASINTSIGNAL)
+_PROTOTYPE(static int lowpgsp,(int sig));
+# else	/* !defined(HASINTSIGNAL) */
+_PROTOTYPE(static void lowpgsp,(int sig));
+# endif	/* defined(HASINTSIGNAL) */
+#endif	/* defined(SIGDANGER) */
+
 
 /*
  * Local definitions
@@ -816,6 +824,14 @@ get_kernel_access()
  * Check the kernel version number.
  */
 	(void) ckkv("AIX", (char *)NULL, LSOF_VSTR, (char *)NULL);
+
+#if	defined(SIGDANGER)
+/*
+ * If SIGDANGER is defined, enable its handler.
+ */
+	(void) signal(SIGDANGER, lowpgsp);
+#endif	/* defined(SIGDANGER) */
+
 }
 
 
@@ -1093,6 +1109,26 @@ kreadx(addr, buf, len, sid)
 }
 
 
+#if	defined(SIGDANGER)
+/*
+ * lowpgsp() - hangle a SIGDANGER signal about low paging space
+*/
+
+# if	defined(HASINTSIGNAL)
+static int
+# else	/* !defined(HASINTSIGNAL) */
+static void
+# endif	/* defined(HASINTSIGNAL) */
+
+lowpgsp(sig)
+	int sig;
+{
+	(void) fprintf(stderr, "%s: FATAL: system paging space is low.\n", Pn);
+	Exit(1);
+}
+#endif	/* defined(SIGDANGER) */
+
+
 #if	AIXA<2
 /*
  * process_text() - process text file information for non-ia64 AIX
