@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.31 2004/03/10 23:50:58 abe Exp $
+ * $Id: machine.h,v 1.32 2004/07/06 19:18:57 abe Exp $
  */
 
 
@@ -40,6 +40,23 @@
 
 
 #include <sys/types.h>
+
+#if	defined(HASCPUMASK_T)
+/*
+ * In FreeBSD >= 5.2 when the cpumask_t typedef is present, it is defined int
+ * <sys/types.h> only if _KERNEL is predefined.  However, predefining _KERNEL
+ * before #include'ing <sys/types.h> causes redefinition errors for boolean_t
+ * and vm_page_t when <vm/vm.h> is #include'd with _KERNEL predefined.  Since
+ * lsof must have _KERNEL predefined when <vm/vm.h> is #include'd, the
+ * expedient choice is made to duplicate the cpumask_t typedef here.
+ *
+ * Note: the Configure script defines HASCPUMASK_T if <sys/types.h> and
+ * <machine/_types.h> both have cpumask_t references.
+ */
+
+typedef	__cpumask_t	cpumask_t;
+#endif	/* defined(HASCPUMASK_T) */
+
 #include <sys/param.h>
 
 
@@ -81,9 +98,9 @@
  * recorded in BDevtp[].
  */
 
-# if	FREEBSDV<400
+# if	FREEBSDV<4000
 #define	HASBLKDEV	1
-# endif	/* FREEBSDV<400 */
+# endif	/* FREEBSDV<4000 */
 
 
 /*
@@ -211,6 +228,14 @@
 /* #define SETLFILEADD Lf->... */
 
 
+/*
+ * HASMNTSUP is defined for those dialects that support the mount supplement
+ * option.
+ */
+
+/* #define	HASMNTSUP	1	*/
+
+
 /*
  * HASMOPT is defined for those dialects that support the reading of
  * kernel memory from an alternate file.
@@ -250,9 +275,9 @@
  * NOTE: don't forget to define a prototype for this function in dproto.h.
  */
 
-# if	FREEBSDV>=220
+# if	FREEBSDV>=2020
 #define	HASPIPEFN	process_pipe
-# endif	/* FREEBSDV>=220 */
+# endif	/* FREEBSDV>=2020 */
 
 
 /*
@@ -275,9 +300,9 @@
  * the parent process IDentifier (PPID) of a process.
  */
 
-# if	FREEBSDV>=200
+# if	FREEBSDV>=2000
 #define	HASPPID		1
-# endif	/* FREEBSDV>=200 */
+# endif	/* FREEBSDV>=2000 */
 
 
 /*
@@ -364,9 +389,9 @@
 /* #define		HASPROCFS	"proc?" */
 /* #define		HASFSTYPE	1 */
 
-# if	FREEBSDV>=200
+# if	FREEBSDV>=2000
 #define	HASPINODEN	1
-# endif	/* FREEBSDV>=200 */
+# endif	/* FREEBSDV>=2000 */
 
 
 /*
@@ -518,11 +543,11 @@
 /* #define	USE_LIB_READMNT			1	   rmnt.c */
 /* #define	USE_LIB_REGEX			1	   regex.c */
 
-# if	FREEBSDV<210
+# if	FREEBSDV<2010
 #define	USE_LIB_RNAM				1	/* rnam.c */
-# else	/* FREEBSDV>=210 */
+# else	/* FREEBSDV>=2010 */
 #define	USE_LIB_RNMH				1	/* rnmh.c */
-# endif	/* FREEBSDV<210 */
+# endif	/* FREEBSDV<2010 */
 
 /* #define	USE_LIB_RNCH			1	   rnch.c */
 /* #define	USE_LIB_SNPF			1	   snpf.c */
