@@ -262,7 +262,7 @@ print_tcptpi(nl)
 	    ps++;
 	}
 
-# if	defined(HASTCPTPIQ)
+#if	defined(HASTCPTPIQ)
 	if (Ftcptpi & TCPTPI_QUEUES) {
 	    if (Lf->lts.rqs) {
 		if (Ffield)
@@ -293,9 +293,292 @@ print_tcptpi(nl)
 		ps++;
 	    }
 	}
-# endif	/* defined(HASTCPTPIQ) */
+#endif	/* defined(HASTCPTPIQ) */
 
-# if	defined(HASTCPTPIW)
+#if	defined(HASSOOPT)
+	if (Ftcptpi & TCPTPI_FLAGS) {
+	    int opt;
+
+	    if ((opt = Lf->lts.opt) || Lf->lts.qlens || Lf->lts.qlims) {
+		char sep = ' ';
+
+		if (Ffield)
+		    sep = LSOF_FID_TCPTPI;
+		else if (!ps)
+		    sep = '(';
+		(void) printf("%cSO", sep);
+		ps++;
+		sep = '=';
+
+# if	defined(PS_SO_ACCEPTCONN)
+		if (opt & PS_SO_ACCEPTCONN) {
+		    (void) printf("%cACCEPTCONN", sep);
+		    opt &= ~PS_SO_ACCEPTCONN;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_ACCEPTCONN) */
+
+# if	defined(PS_SO_BROADCAST)
+		if (opt & PS_SO_BROADCAST) {
+		    (void) printf("%cBROADCAST", sep);
+		    opt &= ~PS_SO_BROADCAST;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_BROADCAST) */
+
+# if	defined(PS_SO_DEBUG)
+		if (opt & PS_SO_DEBUG) {
+		    (void) printf("%cDEBUG", sep);
+		    opt &= ~PS_SO_DEBUG;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_DEBUG) */
+
+# if	defined(PS_SO_DONTROUTE)
+		if (opt & PS_SO_DONTROUTE) {
+		    (void) printf("%cDONTROUTE", sep);
+		    opt &= ~PS_SO_DONTROUTE;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_DONTROUTE) */
+
+# if	defined(PS_SO_GETIFADDR)
+		if (opt & PS_SO_GETIFADDR) {
+		    (void) printf("%cGETIFADDR", sep);
+		    opt &= ~PS_SO_GETIFADDR;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_GETIFADDR) */
+
+# if	defined(PS_SO_INPCB_COPY)
+		if (opt & PS_SO_INPCB_COPY) {
+		    (void) printf("%cINPCB_COPY", sep);
+		    opt &= ~PS_SO_INPCB_COPY;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_INPCB_COPY) */
+
+# if	defined(PS_SO_KEEPALIVE)
+		if (opt & PS_SO_KEEPALIVE) {
+		    (void) printf("%cKEEPALIVE", sep);
+		    if (Lf->lts.kai)
+			(void) printf("=%d", Lf->lts.kai);
+		    opt &= ~PS_SO_KEEPALIVE;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_KEEPALIVE) */
+
+# if	defined(PS_SO_LINGER)
+		if (opt & PS_SO_LINGER) {
+		    (void) printf("%cLINGER", sep);
+		    if (Lf->lts.ltm)
+			(void) printf("=%d", Lf->lts.ltm);
+		    opt &= ~PS_SO_LINGER;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_LINGER) */
+
+# if	defined(PS_SO_OOBINLINE)
+		if (opt & PS_SO_OOBINLINE) {
+		    (void) printf("%cOOBINLINE", sep);
+		    opt &= ~PS_SO_OOBINLINE;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_OOBINLINE) */
+
+# if	defined(PS_SO_PMTU)
+		if (opt & PS_SO_PMTU) {
+		    (void) printf("%cPMTU", sep);
+		    opt &= ~PS_SO_PMTU;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_PMTU) */
+
+		if (Lf->lts.qlens) {
+		    (void) printf("%cQLEN=%d", sep, Lf->lts.qlen);
+		    sep = ',';
+		}
+		if (Lf->lts.qlims) {
+		    (void) printf("%cQLIM=%lu", sep, Lf->lts.qlim);
+		    sep = ',';
+		}
+
+# if	defined(PS_SO_REUSEADDR)
+		if (opt & PS_SO_REUSEADDR) {
+		    (void) printf("%cREUSEADDR", sep);
+		    opt &= ~PS_SO_REUSEADDR;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_REUSEADDR) */
+
+# if	defined(PS_SO_REUSEPORT)
+		if (opt & PS_SO_REUSEPORT) {
+		    (void) printf("%cREUSEPORT", sep);
+		    opt &= ~PS_SO_REUSEPORT;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_REUSEPORT) */
+
+# if	defined(PS_SO_USELOOPBACK)
+		if (opt & PS_SO_USELOOPBACK) {
+		    (void) printf("%cUSELOOPBACK", sep);
+		    opt &= ~PS_SO_USELOOPBACK;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_USELOOPBACK) */
+
+		if (opt)
+		    (void) printf("%cUNKNOWN=%#x", sep, opt);
+		if (Ffield)
+		    putchar(Terminator);
+	    }
+	}
+#endif	/* defined(HASSOOPT) */
+
+#if	defined(HASSOSTATE)
+	if (Ftcptpi & TCPTPI_FLAGS) {
+	    int ss;
+
+	    if ((ss = Lf->lts.ss)) {
+		char sep = ' ';
+
+		if (Ffield)
+		    sep = LSOF_FID_TCPTPI;
+		else if (!ps)
+		    sep = '(';
+		(void) printf("%cSS", sep);
+		ps++;
+		sep = '=';
+
+# if	defined(PS_SS_ASYNC)
+		if (ss & PS_SS_ASYNC) {
+		    (void) printf("%cASYNC", sep);
+		    ss &= ~PS_SS_ASYNC;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_ASYNC) */
+
+# if	defined(PS_SS_BOUND)
+		if (ss & PS_SS_BOUND) {
+		    (void) printf("%cBOUND", sep);
+		    ss &= ~PS_SS_BOUND;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_BOUND) */
+
+# if	defined(PS_SS_CANTRCVMORE)
+		if (ss & PS_SS_CANTRCVMORE) {
+		    (void) printf("%cCANTRCVMORE", sep);
+		    ss &= ~PS_SS_CANTRCVMORE;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_CANTRCVMORE) */
+
+# if	defined(PS_SS_CANTSENDMORE)
+		if (ss & PS_SS_CANTSENDMORE) {
+		    (void) printf("%cCANTSENDMORE", sep);
+		    ss &= ~PS_SS_CANTSENDMORE;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_CANTSENDMORE) */
+
+# if	defined(PS_SS_ISCONNECTED)
+		if (ss & PS_SS_ISCONNECTED) {
+		    (void) printf("%cISCONNECTED", sep);
+		    ss &= ~PS_SS_ISCONNECTED;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_ISCONNECTED) */
+
+# if	defined(PS_SS_ISCONNECTING)
+		if (ss & PS_SS_ISCONNECTING) {
+		    (void) printf("%cISCONNECTING", sep);
+		    ss &= ~PS_SS_ISCONNECTING;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_ISCONNECTING) */
+
+# if	defined(PS_SS_ISDISCONNECTI)
+		if (ss & PS_SS_ISDISCONNECTI) {
+		    (void) printf("%cISDISCONNECTI", sep);
+		    ss &= ~PS_SS_ISDISCONNECTI;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_ISDISCONNECTI) */
+
+# if	defined(PS_SS_INTERRUPTED)
+		if (ss & PS_SS_INTERRUPTED) {
+		    (void) printf("%cINTERRUPTED", sep);
+		    ss &= ~PS_SS_INTERRUPTED;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_INTERRUPTED) */
+
+# if	defined(PS_SS_NBIO)
+		if (ss & PS_SS_NBIO) {
+		    (void) printf("%cNBIO", sep);
+		    ss &= ~PS_SS_NBIO;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_NBIO) */
+
+# if	defined(PS_SS_NOFDREF)
+		if (ss & PS_SS_NOFDREF) {
+		    (void) printf("%cNOFDREF", sep);
+		    ss &= ~PS_SS_NOFDREF;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_NOFDREF) */
+
+# if	defined(PS_SS_NOUSER)
+		if (ss & PS_SS_NOUSER) {
+		    (void) printf("%cNOUSER", sep);
+		    ss &= ~PS_SS_NOUSER;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_NOUSER) */
+
+# if	defined(PS_SS_NOWAIT)
+		if (ss & PS_SS_NOWAIT) {
+		    (void) printf("%cNOWAIT", sep);
+		    ss &= ~PS_SS_NOWAIT;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_NOWAIT) */
+
+# if	defined(PS_SS_PRIV)
+		if (ss & PS_SS_PRIV) {
+		    (void) printf("%cPRIV", sep);
+		    ss &= ~PS_SS_PRIV;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_PRIV) */
+
+# if	defined(PS_SS_RCVATMARK)
+		if (ss & PS_SS_RCVATMARK) {
+		    (void) printf("%cRCVATMARK", sep);
+		    ss &= ~PS_SS_RCVATMARK;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_RCVATMARK) */
+
+# if	defined(PS_SS_XOPEN_EXT1)
+		if (ss & PS_SS_XOPEN_EXT1) {
+		    (void) printf("%cXOPEN_EXT1", sep);
+		    ss &= ~PS_SS_XOPEN_EXT1;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_XOPEN_EXT1) */
+
+		if (ss)
+		    (void) printf("%cUNKNOWN=%#x", sep, ss);
+		if (Ffield)
+		    putchar(Terminator);
+	    }
+	}
+#endif	/* defined(HASSOSTATE) */
+
+#if	defined(HASTCPTPIW)
 	if (Ftcptpi & TCPTPI_WINDOWS) {
 	    if (Lf->lts.rws) {
 		if (Ffield)
@@ -326,7 +609,7 @@ print_tcptpi(nl)
 		ps++;
 	    }
 	}
-# endif	/* defined(HASTCPTPIW) */
+#endif	/* defined(HASTCPTPIW) */
 
 	if (Ftcptpi && !Ffield && ps)
 	    putchar(')');
@@ -416,6 +699,24 @@ process_socket(f, s)
 	}
 #endif  /* defined(HASTCPTPIQ) */
 
+#if	defined(HASSOOPT)
+/*
+ * Enter socket options.
+ */
+	Lf->lts.opt = (unsigned int)s->pst_options;
+	Lf->lts.ltm = (unsigned int)s->pst_linger;
+	Lf->lts.qlen = (unsigned int)s->pst_qlen;
+	Lf->lts.qlim = (unsigned int)s->pst_qlimit;
+	Lf->lts.qlens = Lf->lts.qlims = (unsigned char)1;
+#endif	/* defined(HASSOOPT) */
+
+#if	defined(HASSOSTATE)
+/*
+ * Enter socket state flags.
+ */
+	Lf->lts.ss = (unsigned int)s->pst_state;
+#endif	/* defined(HASSOSTATE) */
+
 #if	defined(HASTCPTPIW)
 /*
  * Enter window sizes.
