@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.14 2003/06/13 09:52:21 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.15 2004/07/06 19:19:44 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -51,8 +51,10 @@ static char *rcsid = "$Id: dproc.c,v 1.14 2003/06/13 09:52:21 abe Exp $";
  */
 
 _PROTOTYPE(static int getlinksrc,(char *ln, char *src, int srcl, int *slash));
-_PROTOTYPE(static void process_proc_map,(char *p, struct stat *s));
-_PROTOTYPE(static int read_proc_stat,(char *p, int pid, char **cmd, int *ppid, int *pgid));
+_PROTOTYPE(static void process_proc_map,(char *p, struct stat *s, int ss));
+_PROTOTYPE(static int read_proc_stat,(char *p, int pid, char **cmd, int *ppid,
+	   int *pgid));
+_PROTOTYPE(static int statEx,(char *p, struct stat *s, int *ss));
 
 
 /*
@@ -64,16 +66,16 @@ gather_proc_info()
 {
 	static char *cmd = (char *)NULL;
 	char *cp;
-	short cscko, pss, sf, scko;
+	short cscko, lnk, pn, pss, sf, scko;
 	struct dirent *dp, *fp;
 	static char *dpath = (char *)NULL;
 	static int dpathl = 0;
-	int f, fd, i, n, slash, sv;
+	int enls, enss, f, fd, i, ls, n, slash, ss, sv;
 	DIR *fdp;
 	struct stat lsb, sb;
+	char nmabuf[MAXPATHLEN + 1], pbuf[MAXPATHLEN + 1];
 	static char *path = (char *)NULL;
 	static int pathl = 0;
-	char pbuf[MAXPATHLEN + 1];
 	int pgid, pid, ppid;
 	static char *pidpath = (char *)NULL;
 	static MALLOC_S pidpathl = 0;
@@ -213,66 +215,157 @@ gather_proc_info()
 	 * Process the PID's current working directory info.
 	 */
 	    (void) make_proc_path(pidpath, n, &path, &pathl, "cwd");
-	    if (getlinksrc(path, pbuf, sizeof(pbuf), &slash) > 0) {
+	    alloc_lfile(CWD, -1);
+	    if (getlinksrc(path, pbuf, sizeof(pbuf), &slash) < 1) {
+		if (!Fwarn) {
+		    (void) memset((void *)&sb, 0, sizeof(sb));
+		    lnk = ss = 0;
+		    (void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
+			strerror(errno));
+		    nmabuf[sizeof(nmabuf) - 1] = '\0';
+		    (void) add_nma(nmabuf, strlen(nmabuf));
+		    pn = 1;
+		} else
+		    pn = 0;
+	    } else {
+		lnk = 1;
 		if ((scko && !slash) || !scko) {
-		    if (HasNFS)
-			sv = statsafely(path, &sb);
-		    else
+		    pn = 1;
+		    ss = SB_ALL;
+		    if (HasNFS) {
+			if (sv = statsafely(path, &sb)) {
+			    if (lnk)
+				sv = statEx(pbuf, &sb, &ss);
+			}
+		    } else
 			sv = stat(path, &sb);
-		    if (!sv) {
-			alloc_lfile(CWD, -1);
-			(void) process_proc_node(pbuf, &sb,
-						 (struct stat *)NULL);
-			if (Lf->sf)
-			    link_lfile();
+		    if (sv) {
+			ss = 0;
+			if (!Fwarn) {
+			    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
+				strerror(errno));
+			    nmabuf[sizeof(nmabuf) - 1] = '\0';
+			    (void) add_nma(nmabuf, strlen(nmabuf));
+			}
 		    }
-		}
+		} else
+		    pn = 0;
+	    }
+	    if (pn) {
+		(void) process_proc_node(lnk ? pbuf : path,
+					 &sb, ss,
+					 (struct stat *)NULL, 0);
+		if (Lf->sf)
+		    link_lfile();
 	    }
 	/*
 	 * Process the PID's root directory info.
 	 */
 	    (void) make_proc_path(pidpath, n, &path, &pathl, "root");
-	    if (getlinksrc(path, pbuf, sizeof(pbuf), &slash) > 0) {
+	    alloc_lfile(RTD, -1);
+	    if (getlinksrc(path, pbuf, sizeof(pbuf), &slash) < 1) {
+		if (!Fwarn) {
+		    (void) memset((void *)&sb, 0, sizeof(sb));
+		    lnk = ss = 0;
+		    (void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
+			strerror(errno));
+		    nmabuf[sizeof(nmabuf) - 1] = '\0';
+		    (void) add_nma(nmabuf, strlen(nmabuf));
+		    pn = 1;
+		} else
+		    pn = 0;
+	    } else {
+		lnk = 1;
 		if ((scko && !slash) || !scko) {
-		    if (HasNFS)
-			sv = statsafely(path, &sb);
-		    else
+		    pn = 1;
+		    if (HasNFS) {
+			if (sv = statsafely(path, &sb)) {
+			    if (lnk)
+				sv = statEx(pbuf, &sb, &ss);
+			}
+		    } else
 			sv = stat(path, &sb);
-		    if (!sv) {
-			alloc_lfile(RTD, -1);
-			(void) process_proc_node(pbuf, &sb,
-						 (struct stat *)NULL);
-			if (Lf->sf)
-			    link_lfile();
-		    }
-		}
+		    if (sv) {
+			ss = 0;
+			if (!Fwarn) {
+			    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
+				strerror(errno));
+			    nmabuf[sizeof(nmabuf) - 1] = '\0';
+			    (void) add_nma(nmabuf, strlen(nmabuf));
+			}
+		    } else
+			ss = SB_ALL;
+		} else
+		    pn = 0;
+	    }
+	    if (pn) {
+		(void) process_proc_node(lnk ? pbuf : path,
+					 &sb, ss,
+					 (struct stat *)NULL, 0);
+		if (Lf->sf)
+		    link_lfile();
 	    }
 	/*
 	 * Process the PID's execution info.
 	 */
 	    (void) make_proc_path(pidpath, n, &path, &pathl, "exe");
-	    if (getlinksrc(path, pbuf, sizeof(pbuf), &slash) > 0) {
-		if ((scko && !slash) || !scko) {
-		    if (HasNFS)
-			sv = statsafely(path, &sb);
-		    else
-			sv = stat(path, &sb);
-		    if (!sv) {
-			txts = 1;
-			alloc_lfile("txt", -1);
-			(void) process_proc_node(pbuf, &sb,
-						 (struct stat *)NULL);
-			if (Lf->sf)
-			    link_lfile();
+	    alloc_lfile("txt", -1);
+	    if (getlinksrc(path, pbuf, sizeof(pbuf), &slash) < 1) {
+		(void) memset((void *)&sb, 0, sizeof(sb));
+		lnk = ss = 0;
+		if (!Fwarn) {
+		    if ((errno != ENOENT) || uid) {
+			(void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
+			    strerror(errno));
+			nmabuf[sizeof(nmabuf) - 1] = '\0';
+			(void) add_nma(nmabuf, strlen(nmabuf));
 		    }
-		}
+		    pn = 1;
+		} else
+		    pn = 0;
+	    } else {
+		lnk = 1;
+		if ((scko && !slash) || !scko) {
+		    pn = 1;
+		    if (HasNFS) {
+			if (sv = statsafely(path, &sb)) {
+			    if (lnk) {
+				sv = statEx(pbuf, &sb,  &ss);
+				if (!sv && (ss & SB_DEV) && (ss & SB_INO))
+				    txts = 1;
+			    }
+			}
+		    } else
+			sv = stat(path, &sb);
+		    if (sv) {
+			ss = 0;
+			if (!Fwarn) {
+			    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
+				strerror(errno));
+			    nmabuf[sizeof(nmabuf) - 1] = '\0';
+			    (void) add_nma(nmabuf, strlen(nmabuf));
+			}
+		    } else {
+			ss = SB_ALL;
+			txts = 1;
+		    }
+		} else
+		    pn = 0;
+	    }
+	    if (pn) {
+		(void) process_proc_node(lnk ? pbuf : path,
+					 &sb, ss,
+					 (struct stat *)NULL, 0);
+		if (Lf->sf)
+		    link_lfile();
 	    }
 	/*
 	 * Process the PID's memory map info.
 	 */
 	    if (!scko) {
 		(void) make_proc_path(pidpath, n, &path, &pathl, "maps");
-		(void) process_proc_map(path, txts ? &sb : (struct stat *)NULL);
+		(void) process_proc_map(path, txts ? &sb : (struct stat *)NULL,
+					txts ? ss : 0);
 	    }
 	/*
 	 * Process the PID's file descriptor directory.
@@ -280,8 +373,17 @@ gather_proc_info()
 	    if ((i = make_proc_path(pidpath, n, &dpath, &dpathl, "fd/")) < 3)
 		continue;
 	    dpath[i - 1] = '\0';
-	    if (!(fdp = opendir(dpath)))
+	    if (!(fdp = opendir(dpath))) {
+		if (!Fwarn) {
+		    (void) snpf(nmabuf, sizeof(nmabuf), "%s (opendir: %s)",
+			dpath, strerror(errno));
+		    alloc_lfile("NOFD", -1);
+		    nmabuf[sizeof(nmabuf) - 1] = '\0';
+		    (void) add_nma(nmabuf, strlen(nmabuf));
+		    link_lfile();
+		}
 		continue;
+	    }
 	    dpath[i - 1] = '/';
 	    while ((fp = readdir(fdp))) {
 		for (f = fd = n = 0, cp = fp->d_name; *cp; cp++) {
@@ -302,21 +404,65 @@ gather_proc_info()
 		if (f)
 		    continue;
 		(void) make_proc_path(dpath, i, &path, &pathl, fp->d_name);
-		if (getlinksrc(path, pbuf, sizeof(pbuf), &slash) > 0) {
+		(void) alloc_lfile((char *)NULL, fd);
+		if (getlinksrc(path, pbuf, sizeof(pbuf), &slash) < 1) {
+		    (void) memset((void *)&sb, 0, sizeof(sb));
+		    lnk = ss = 0;
+		    if (!Fwarn) {
+			(void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
+			    strerror(errno));
+			nmabuf[sizeof(nmabuf) - 1] = '\0';
+			(void) add_nma(nmabuf, strlen(nmabuf));
+			pn = 1;
+		    } else
+			pn = 0;
+		} else {
+		    lnk = 1;
 		    if ((scko && !slash) || !scko) {
+			pn = 1;
 			if (HasNFS) {
-			    if (statsafely(path, &sb)
-			    ||  lstatsafely(path, &lsb))
-				continue;
+			    if (lstatsafely(path, &lsb)) {
+				if (lnk)
+				    (void) statEx(pbuf, &lsb, &ls);
+				else
+				    ls = 0;
+				enls = errno;
+			    } else
+				ls = SB_ALL;
+			    if (statsafely(path, &sb)) {
+				if (lnk)
+				    (void) statEx(pbuf, &sb, &ss);
+				else
+				    ss = SB_ALL;
+				enss = errno;
+			    } else
+				ss = SB_ALL;
 			} else {
-			    if (stat(path, &sb) || lstat(path, &lsb))
-				continue;
+			    ls = lstat(path, &lsb) ? 0 : SB_ALL;
+			    enls = errno;
+			    ss = stat(path, &sb) ? 0 : SB_ALL;
+			    enss = errno;
 			}
-			(void) alloc_lfile((char *)NULL, fd);
-			process_proc_node(pbuf, &sb, &lsb);
-			if (Lf->sf)
-			    link_lfile();
-		    }
+			if (!ls && !Fwarn) {
+			    (void) snpf(nmabuf, sizeof(nmabuf), "lstat: %s)",
+				strerror(enls));
+			    nmabuf[sizeof(nmabuf) - 1] = '\0';
+			    (void) add_nma(nmabuf, strlen(nmabuf));
+			}
+			if (!ss && !Fwarn) {
+			    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
+				strerror(enss));
+			    nmabuf[sizeof(nmabuf) - 1] = '\0';
+			    (void) add_nma(nmabuf, strlen(nmabuf));
+			}
+		    } else
+			pn = 0;
+		}
+		if (pn) {
+		    process_proc_node(lnk ? pbuf : path,
+				      &sb, ss, &lsb, ls);
+		    if (Lf->sf)
+			link_lfile();
 		}
 	    }
 	    (void) closedir(fdp);
@@ -455,13 +601,14 @@ make_proc_path(pp, pl, np, nl, sf)
  */
 
 static void
-process_proc_map(p, s)
+process_proc_map(p, s, ss)
 	char *p;			/* path to process maps file */
-	struct stat *s;			/* exexuting text file state buffer */
+	struct stat *s;			/* executing text file state buffer */
+	int ss;				/* *s status -- i.e., SB_* values */
 {
-	char buf[MAXPATHLEN], *ep, **fp;
-	int del, i, nf, sv;
+	char buf[MAXPATHLEN + 1], *ep, **fp, nmabuf[MAXPATHLEN + 1];
 	dev_t dev;
+	int en, i, mss, nf, sv;
 	ino_t inode;
 	MALLOC_S len;
 	long maj, min;
@@ -508,7 +655,8 @@ process_proc_map(p, s)
 	 * If they do, skip this map entry.
 	 */
 	    dev = (dev_t)makedev((int)maj, (int)min);
-	    if (s && dev == s->st_dev && inode == s->st_ino)
+	    if (s && (ss & SB_DEV) && (ss & SB_INO)
+	    &&  (dev == s->st_dev) && (inode == s->st_ino))
 		continue;
 	/*
 	 * See if this device + inode pair has already been processed as
@@ -540,46 +688,83 @@ process_proc_map(p, s)
 	    sm[ns].dev = dev;
 	    sm[ns++].inode = inode;
 	/*
-	 * Get a stat(2) buffer for the mapped file.  If that succeeds,
-	 * allocate space for the file and process it.
+	 * Allocate space for the mapped file, then get stat(2) information
+	 * for it.
 	 */
-	    del = 0;
-	    if (HasNFS)
+	    alloc_lfile("mem", -1);
+	    if (HasNFS) {
 		sv = statsafely(fp[6], &sb);
-	    else
+	    } else
 		sv = stat(fp[6], &sb);
 	    if (sv) {
+		en = errno;
+	    /*
+	     * Applying stat(2) to the file failed, so manufacture a partial
+	     * stat(2) reply from the process' maps file entry.
+	     *
+	     * If the file has been deleted, reset its type to "DEL"; otherwise
+	     * generate a stat() error name addition.
+	     */
+		(void) memset((void *)&sb, 0, sizeof(sb));
+		sb.st_dev = dev;
+		sb.st_ino = inode;
+		sb.st_mode = S_IFREG;
+		mss = SB_DEV | SB_INO | SB_MODE;
+		if ((nf >= 8) && !strcmp(fp[7], "(deleted)"))
+		    alloc_lfile("DEL", -1);
+		else {
+		    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
+			strerror(en));
+		    nmabuf[sizeof(nmabuf) - 1] = '\0';
+		    (void) add_nma(nmabuf, strlen(nmabuf));
+		}
+	    } else if ((sb.st_dev != dev) || (sb.st_ino != inode)) {
 
 	    /*
-	     * Applying stat(2) to the file failed.  See if the file has been
-	     * deleted.  If it has, manufacture a partial stat(2) reply, and
-	     * remember the deletion status.
+	     * The stat(2) device and inode numbers don't match those obtained
+	     * from the process' maps file.
+	     *
+	     * If the file has been deleted, reset its type to "DEL"; otherwise
+	     * generate inconsistency name additions.
+	     *
+	     * Manufacture a partial stat(2) reply from the maps file
+	     * information.
 	     */
-		if ((nf >= 8) && !strcmp(fp[7], "(deleted)")) {
-		    (void) memset((void *)&sb, 0, sizeof(sb));
-		    sb.st_dev = dev;
-		    sb.st_ino = inode;
-		    sb.st_mode = S_IFREG;
-		    del = 1;
-		    sv = 0;
-		}
-	    }
-	    if (!sv) {
-		alloc_lfile("mem", -1);
-		process_proc_node(fp[6], &sb, (struct stat *)NULL);
-		if (Lf->sf) {
-		    if (del) {
+		if ((nf >= 8) && !strcmp(fp[7], "(deleted)"))
+		    alloc_lfile("DEL", -1);
+		else if (!Fwarn) {
+		    char *sep;
 
-		    /*
-		     * If the stat(2) buffer was manufactured, change some
-		     * local file structure items.
-		     */
-			Lf->sz_def = 0;
-			(void) snpf(Lf->type, sizeof(Lf->type), "%s", "DEL");
+		    if (sb.st_dev != dev) {
+			(void) snpf(nmabuf, sizeof(nmabuf),
+			    "(path dev=%d,%d%s",
+			    GET_MAJ_DEV(sb.st_dev), GET_MIN_DEV(sb.st_dev),
+			    (sb.st_ino == inode) ? ")" : "");
+			nmabuf[sizeof(nmabuf) - 1] = '\0';
+			(void) add_nma(nmabuf, strlen(nmabuf));
+			sep = "";
+		    } else
+			sep = "(path ";
+		    if (sb.st_ino != inode) {
+			(void) snpf(nmabuf, sizeof(nmabuf),
+			    "%sinode=%ld)", sep, sb.st_ino);
+			nmabuf[sizeof(nmabuf) - 1] = '\0';
+			(void) add_nma(nmabuf, strlen(nmabuf));
 		    }
-		    link_lfile();
 		}
-	    }
+		(void) memset((void *)&sb, 0, sizeof(sb));
+		sb.st_dev = dev;
+		sb.st_ino = inode;
+		sb.st_mode = S_IFREG;
+		mss = SB_DEV | SB_INO | SB_MODE;
+	    } else
+		mss = SB_ALL;
+	/*
+	 * Record the file's information.
+	 */
+	    process_proc_node(fp[6], &sb, mss, (struct stat *)NULL, 0);
+	    if (Lf->sf)
+		link_lfile();
 	}
 	(void) fclose(ms);
 }
@@ -614,12 +799,13 @@ read_proc_stat(p, pid, cmd, ppid, pgid)
 	    return(1);
 /*
  * Separate the line into fields on white space separators.
- *
- * Convert the first field to an integer; its conversion must match the
- * PID argument.
  */
 	if ((nf = get_fields(buf, (char *)NULL, &fp)) < 5)
 	    return(1);
+/*
+ * Convert the first field to an integer; its conversion must match the
+ * PID argument.
+ */
 	if (atoi(fp[0]) != pid)
 	    return(1);
 /*
@@ -649,7 +835,14 @@ read_proc_stat(p, pid, cmd, ppid, pgid)
 	(void) snpf(cbf, len, "%s", cp);
 	*cmd = cbf;
 /*
- * Convert and return parent process and process group IDs.
+ * Check the state in the third field.  If it is 'Z', this is a defunct
+ * process and should be skipped.
+ */
+	if (!fp[2] || !strcmp(fp[2], "Z"))
+	    return(1);
+/*
+ * Convert and return parent process (fourth field) and process group (fifth
+ * field) IDs.
  */
 	if (fp[3] && *fp[3])
 	    *ppid = atoi(fp[3]);
@@ -661,3 +854,82 @@ read_proc_stat(p, pid, cmd, ppid, pgid)
 	    return(1);
 	return(0);
 }
+
+
+/*
+ * statEx() - extended stat() to get device numbers when a "safe" stat has
+ *	      failed and the system has an NFS mount
+ *
+ * Note: this function was suggested by Paul Szabo as a way to get device
+ *       numbers for NFS files when an NFS mount point has the root_squash
+ *       option set.  In that case, even if lsof is setuid(root), the identity
+ *	 of its requests to stat() NFS files lose root permission and may fail.
+ *
+ *	 This function should be used only when links have been successfully
+ *	 resolved in the /proc path by getlinksrc().
+ */
+
+static int
+statEx(p, s, ss)
+	char *p;			/* file path */
+	struct stat *s;			/* stat() result -- NULL if none
+					 * wanted */
+	int *ss;			/* stat() status --  SB_* values */
+{
+	static size_t ca = 0;
+	static char *cb = NULL;
+	char *cp;
+	int ensv = ENOENT;
+	struct stat sb;
+	int st = 0;
+	size_t sz;
+/*
+ * Make a copy of the path.
+ */
+	sz = strlen(p);
+	if ((sz + 1) > ca) {
+	    if (cb)
+		cb = (char *)realloc((MALLOC_P *)cb, sz + 1);
+	    else
+		cb = (char *)malloc(sz + 1);
+	    if (!cb) {
+		(void) fprintf(stderr,
+		    "%s: PID %ld: no statEx path space: %s\n",
+		    Pn, (long)Lp->pid, p);
+		Exit(1);
+	    }
+	    ca = sz + 1;
+	}
+	(void) strcpy(cb, p);
+/*
+ * Trim trailing leaves from the end of the path one at a time and do s safe
+ * stat() on each trimmed result.  Stop when a safe stat() succeeds or doesn't
+ * fail because of EACCES or EPERM.
+ */
+	for (cp = strrchr(cb, '/'); cp && (cp != cb);) {
+	    *cp = '\0';
+	    if (!statsafely(cb, &sb)) {
+		st = 1;
+		break;
+	    }
+	    ensv = errno;
+	    if ((ensv != EACCES) && (ensv != EPERM))
+		break;
+	    cp = strrchr(cb, '/');
+	}
+/*
+ * If a stat() on a trimmed result succeeded, form partial results containing
+ * only the device and raw device numbers.
+ */
+	memset((void *)s, 0, sizeof(struct stat));
+	if (st) {
+	    errno = 0;
+	    s->st_dev = sb.st_dev;
+	    s->st_rdev = sb.st_rdev;
+	    *ss = SB_DEV | SB_RDEV;
+	    return(0);
+	}
+	errno = ensv;
+	*ss = 0;
+	return(1);
+}
