@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.24 2004/01/14 11:29:04 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.25 2004/07/06 19:19:44 abe Exp $";
 #endif
 
 
@@ -1396,7 +1396,7 @@ net6a2in6(as, ad)
 	    buf[8] = '\0';
 	    ep = (char *)NULL;
 	    if ((ad->s6_addr32[i] = (uint32_t)strtoul(buf, &ep, 16))
-	    ==  ULONG_MAX || !ep || *ep)
+	    ==  (uint32_t)UINT32_MAX || !ep || *ep)
 		break;
 	}
 	return((*as || (i != 4) || len) ? 1 : 0);
@@ -1628,11 +1628,13 @@ print_tcptpi(nl)
  */
 
 void
-process_proc_sock(p, s, l)
+process_proc_sock(p, s, ss, l, lss)
 	char *p;			/* node's readlink() path */
 	struct stat *s;			/* stat() result for path */
+	int ss;				/* *s status -- i.e, SB_* values */
 	struct stat *l;			/* lstat() result for FD (NULL for
 					 * others) */
+	int lss;				/* *l status -- i.e, SB_* values */
 {
 	struct ax25sin *ap;
 	char *cp, dev_ch[32], *path;
@@ -1654,14 +1656,17 @@ process_proc_sock(p, s, l)
  * Enter offset, if possible.
  */
 	if (Foffset || !Fsize) {
-	    if (l)
+	    if (l && (lss & SB_SIZE)) {
 		Lf->off = (SZOFFTYPE)l->st_size;
-	    Lf->off_def = 1;
+		Lf->off_def = 1;
+	    }
 	}
 /*
  * Check for socket's inode presence in the protocol info caches.
  */
-	if ((tp = check_tcpudp((unsigned long)s->st_ino, &pr))) {
+	if ((ss & SB_INO)
+	&&  (tp = check_tcpudp((unsigned long)s->st_ino, &pr))
+	) {
 
 	/*
 	 * The inode is connected to an IPv4 TCP or UDP /proc record.
@@ -1682,8 +1687,10 @@ process_proc_sock(p, s, l)
 
 	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), "%.*s", IPROTOL-1, pr);
 	    Lf->inp_ty = 2;
-	    (void) snpf(dev_ch, sizeof(dev_ch), "%ld", (long)s->st_ino);
-	    enter_dev_ch(dev_ch);
+	    if (ss & SB_INO) {
+		(void) snpf(dev_ch, sizeof(dev_ch), "%ld", (long)s->st_ino);
+		enter_dev_ch(dev_ch);
+	    }
 	    if (tp->faddr || tp->fport) {
 		fs.s_addr = tp->faddr;
 		fa = (unsigned char *)&fs;
@@ -1708,7 +1715,9 @@ process_proc_sock(p, s, l)
 	}
 
 #if	defined(HASIPv6)
-	if ((tp6 = check_tcpudp6((unsigned long)s->st_ino, &pr))) {
+	if ((ss & SB_INO)
+	&&  (tp6 = check_tcpudp6((unsigned long)s->st_ino, &pr))
+	) {
 
 	/*
 	 * The inode is connected to an IPv6 TCP or UDP /proc record.
@@ -1723,8 +1732,10 @@ process_proc_sock(p, s, l)
 	    (void) snpf(Lf->type, sizeof(Lf->type), "IPv6");
 	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), "%.*s", IPROTOL-1, pr);
 	    Lf->inp_ty = 2;
-	    (void) snpf(dev_ch, sizeof(dev_ch), "%ld", (long)s->st_ino);
-	    enter_dev_ch(dev_ch);
+	    if (ss && SB_INO) {
+		(void) snpf(dev_ch, sizeof(dev_ch), "%ld", (long)s->st_ino);
+		enter_dev_ch(dev_ch);
+	    }
 	    af = AF_INET6;
 	    if (!IN6_IS_ADDR_UNSPECIFIED(&tp6->faddr) || tp6->fport)
 		fa = (unsigned char *)&tp6->faddr;
@@ -1756,7 +1767,9 @@ process_proc_sock(p, s, l)
 	}
 #endif	/* defined(HASIPv6) */
 
-	if ((up = check_unix((unsigned long)s->st_ino))) {
+	if ((ss & SB_INO)
+	&&  (up = check_unix((unsigned long)s->st_ino))
+	) {
 
 	/*
 	 * The inode is connected to a UNIX /proc record.
@@ -1769,18 +1782,22 @@ process_proc_sock(p, s, l)
 	    (void) snpf(Lf->type, sizeof(Lf->type), "unix");
 	    if (up->pcb)
 		enter_dev_ch(up->pcb);
-	    Lf->inode = (unsigned long)s->st_ino;
-	    Lf->inp_ty = 1;
+	    if (ss & SB_INO) {
+		Lf->inode = (unsigned long)s->st_ino;
+		Lf->inp_ty = 1;
+	    }
 	    path = up->path ? up->path : p;
 	    (void) enter_nm(path);
 	    if (Sfile && is_file_named(path,
-				((s->st_mode & S_IFMT) == S_IFCHR) ? 1 : 0))
+		((ss & SB_MODE) && ((s->st_mode & S_IFMT) == S_IFCHR)) ? 1 : 0))
 	    {
 		Lf->sf |= SELNM;
 	    }
 	    return;
 	}
-	if ((ap = check_ax25((unsigned long)s->st_ino))) {
+	if ((ss & SB_INO)
+	&&  (ap = check_ax25((unsigned long)s->st_ino))
+	) {
 	
 	/*
 	 * The inode is connected to an AX25 /proc record.
@@ -1797,7 +1814,9 @@ process_proc_sock(p, s, l)
 	    print_ax25info(ap);
 	    return;
 	}
-	if ((ip = check_ipx((unsigned long)s->st_ino))) {
+	if ((ss & SB_INO)
+	&&  (ip = check_ipx((unsigned long)s->st_ino))
+	) {
 
 	/*
 	 * The inode is connected to an IPX /proc record.
@@ -1806,10 +1825,14 @@ process_proc_sock(p, s, l)
 	 * the addresses, queue sizes, and state in the NAME column.
 	 */
 	    (void) snpf(Lf->type, sizeof(Lf->type), "ipx");
-	    Lf->inode = (unsigned long)s->st_ino;
-	    Lf->inp_ty = 1;
-	    Lf->dev = s->st_dev;
-	    Lf->dev_def = 1;
+	    if (ss & SB_INO) {
+		Lf->inode = (unsigned long)s->st_ino;
+		Lf->inp_ty = 1;
+	    }
+	    if (ss & SB_DEV) {
+		Lf->dev = s->st_dev;
+		Lf->dev_def = 1;
+	    }
 	    cp = Namech;
 	    nl = MAXPATHLEN - 2;
 	    if (ip->la && nl) {
@@ -1846,7 +1869,9 @@ process_proc_sock(p, s, l)
 		enter_nm(Namech);
 	    return;
 	}
-	if ((rp = check_raw((unsigned long)s->st_ino))) {
+	if ((ss & SB_INO)
+	&&  (rp = check_raw((unsigned long)s->st_ino))
+	) {
 
 	/*
 	 * The inode is connected to a raw /proc record.
@@ -1855,8 +1880,10 @@ process_proc_sock(p, s, l)
 	 * address, remote address, and state in the NAME column.
 	 */
 	    (void) snpf(Lf->type, sizeof(Lf->type), "raw");
-	    Lf->inode = (unsigned long)s->st_ino;
-	    Lf->inp_ty = 1;
+	    if (ss & SB_INO) {
+		Lf->inode = (unsigned long)s->st_ino;
+		Lf->inp_ty = 1;
+	    }
 	    cp = Namech;
 	    nl = MAXPATHLEN - 2;
 	    if (rp->la && rp->lal) {
@@ -1905,9 +1932,13 @@ process_proc_sock(p, s, l)
  * The socket's protocol can't be identified.
  */
 	(void) snpf(Lf->type, sizeof(Lf->type), "sock");
-	Lf->inode = (unsigned long)s->st_ino;
-	Lf->inp_ty = 1;
-	Lf->dev = s->st_dev;
-	Lf->dev_def = 1;
+	if (ss & SB_INO) {
+	    Lf->inode = (unsigned long)s->st_ino;
+	    Lf->inp_ty = 1;
+	}
+	if (ss & SB_DEV) {
+	    Lf->dev = s->st_dev;
+	    Lf->dev_def = 1;
+	}
 	enter_nm("can't identify protocol");
 }
