@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.30 2003/06/11 11:44:50 abe Exp $
+ * $Id: dlsof.h,v 1.34 2004/03/10 23:51:34 abe Exp $
  */
 
 
@@ -62,6 +62,7 @@
 #define	_KERNEL
 #  if	defined(NETBSDV)
 struct buf;	/* dummy for function prototype in <sys/buf.h> */
+struct uio;	/* dummy for function prototype in <sys/buf.h> */
 #  endif	/* defined(NETBSDV) */
 #include <sys/ucred.h>
 # endif	/* (defined(OPENBSDV) && OPENBSDV>=3030)
@@ -104,8 +105,13 @@ struct buf;	/* dummy for function prototype in <sys/buf.h> */
 #include <sys/socket.h>
 
 # if	defined(HASMSDOSFS)
+#  if	HASMSDOSFS==1
 #include <msdosfs/bpb.h>
 #include <msdosfs/fat.h>
+#  else	/* HASMSDOSFS!=1 */
+#include <fs/msdosfs/bpb.h>
+#include <fs/msdosfs/fat.h>
+#  endif	/* HASMSDOSFS==1 */
 
 #  if	(defined(OPENBSDV) && OPENBSDV<3030) \
    ||	(defined(NETBSDV) && __NetBSD_Version__<106060000)
@@ -163,6 +169,26 @@ struct nameidata;	/* to satisfy a function prototype in msdosfsmount.h */
 #include <netinet/tcp_fsm.h>
 #include <netinet/tcp_timer.h>
 #include <netinet/tcp_var.h>
+
+# if	defined(OPENBSDV)
+#  if	!defined(TF_ECN_PERMIT)
+#define	TF_ECN_PERMIT	0x00008000	/* other side said I could ECN */
+#  endif	/* !defined(TF_ECN_PERMIT) */
+
+#  if	!defined(TF_RCVD_CE)
+#define	TF_RCVD_CE	0x00010000	/* send ECE in subsequent segs */ 
+#  endif	/* !defined(TF_RCVD_CE) */
+
+#  if	!defined(TF_SEND_CWR)
+#define	TF_SEND_CWR	0x00020000	/* send CWR in next seg */
+#  endif	/* !defined(TF_SEND_CWR) */
+
+#  if	!defined(TF_DISABLE_ECN)
+#define	TF_DISABLE_ECN	0x00040000	/* disable ECN for this connection */
+#  endif	/* !defined(TF_DISABLE_ECN) */
+
+# endif	/* defined(OPENBSDV) */
+
 #include <sys/ucred.h>
 
 # if	defined(UVM)
@@ -234,14 +260,31 @@ struct sockproto {
 #include <sys/proc.h>
 #include <kvm.h>
 #include <sys/sysctl.h>
+#  if	defined(HASKVMGETPROC2)
+#define	P_ADDR		p_paddr
+#define	P_COMM		p_comm
+#define	P_CWDI		p_cwdi
+#define	P_FD		p_fd
+#define	P_PID		p_pid
+#define	P_PGID		p__pgid
+#define	P_PPID		p_ppid
+#define	P_STAT		p_stat
+#define	P_TRACEP	p_tracep
+#define	P_UID		p_uid
+#define	P_VMSPACE	p_vmspace
+#  else	/* !defined(HASKVMGETPROC2) */
 #define	P_ADDR		kp_eproc.e_paddr
 #define	P_COMM		kp_proc.p_comm
+#define	P_CWDI		kp_proc.p_cwdi
 #define	P_FD		kp_proc.p_fd
 #define	P_PID		kp_proc.p_pid
 #define	P_PGID		kp_eproc.e_pgid
 #define	P_PPID		kp_eproc.e_ppid
 #define	P_STAT		kp_proc.p_stat
+#define	P_TRACEP	kp_proc.p_tracep
+#define	P_UID		kp_eproc.e_ucred.cr_uid
 #define	P_VMSPACE	kp_proc.p_vmspace
+#  endif	/* defined(HASKVMGETPROC2) */
 
 # if	defined(HASFDESCFS)
 #define	_KERNEL
@@ -251,8 +294,39 @@ struct sockproto {
 
 # if	defined(HASKERNFS)
 #define	_KERNEL
+#define	Pkern		__Pkern
+#define	Proot		__Proot
+#define	Pnull		__Pnull
+#define	Ptime		__Ptime
+#define	Pint		__Pint
+#define	Pstring		__Pstring
+#define	Phostname	__Phostname
+#define	Pavenrun	__Pavenrun
+#define	Pdevice		__Pdevice
+#define	Pmsgbuf		__Pmsgbuf
+#define	Pipsecsadir	__Pipsecsadir
+#define	Pipsecspdir	__Pipsecspdir
+#define	Pipsecsa	__Pipseca
+#define	Pipsecsp	__Pipsecsp
 #include <miscfs/kernfs/kernfs.h>
 #undef	_KERNEL
+#undef	Pkern
+#undef	Proot
+#undef	Pnull
+#undef	Ptime
+#undef	Pint
+#undef	Pstring
+#undef	Phostname
+#undef	Pavenrun
+#undef	Pdevice
+#undef	Pmsgbuf
+#undef	Pipsecsadir
+#undef	Pipsecspdir
+#undef	Pipseca
+#undef	Pipsecsp
+#  if	defined(HASKERNFS_KFS_KT)
+#define	kf_kt	kfs_kt
+#  endif	/* defined(HASKERNFS_KFS_KT) */
 # endif	/* defined(HASKERNFS) */
 
 # if	defined(HASNULLFS)
@@ -266,7 +340,24 @@ struct sockproto {
 # endif	/* defined(HASNULLFS) */
 
 # if	defined(HASPROCFS)
+#  if	defined(HASPROCFS_PFSROOT)
+#define	_KERNEL
+#  endif	/* defined(HASPROCFS_PFSROOT) */
 #include <miscfs/procfs/procfs.h>
+#  if	defined(HASPROCFS_PFSROOT)
+#undef	_KERNEL
+#define	Proot		PFSroot
+#define	Pproc		PFSproc
+#define	Pcurproc	PFScurproc
+#define	Pmem		PFSmem
+#define	Pregs		PFSregs
+#define	Pfile		PFSfile
+#define	Pfpregs		PFSfpregs
+#define	Pctl		PFSctl
+#define	Pstatus		PFSstatus
+#define	Pnote		PFSnote
+#define	Pnotepg		PFSnotepg
+#  endif	/* defined(HASPROCFS_PFSROOT) */
 #include <machine/reg.h>
 # endif	/* defined(HASPROCFS) */
 
@@ -386,7 +477,13 @@ struct mounts {
 #define	NL_NAME		n_name
 
 extern int Np;				/* number of kernel processes */
+
+# if	defined(HASKVMGETPROC2) 
+struct kinfo_proc2 *P;			/* local process table copy */
+# else	/* ! defined(HASKVMGETPROC2) */
 struct kinfo_proc *P;			/* local process table copy */
+# endif	/* defined(HASKVMGETPROC2) */
+
 extern int pgshift;			/* kernel's page shift */
 
 struct sfile {
@@ -414,7 +511,7 @@ struct sfile {
 
 
 /*
- * Definitions for rnam.c
+ * Definitions for rnam.c and rnmh.c
  */
 
 # if     defined(HASNCACHE)
