@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.32 2004/01/14 11:29:17 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.33 2004/07/06 19:20:18 abe Exp $";
 #endif
 
 
@@ -745,7 +745,13 @@ process_overlaid_node:
 #if	defined(HASKERNFS)
 	case KERNFSNODE:
 	    if (vfs) {
+
+# if	defined(HASSTATVFS)
+		dev = (dev_t)vfs->fsid.__fsid_val[0];
+# else	/* !defined(HASSTATVFS) */
 		dev = (dev_t)vfs->fsid.val[0];
+# endif	/* defined(HASSTATVFS) */
+
 		devs = 1;
 	    }
 	    break;
@@ -1066,10 +1072,15 @@ process_overlaid_node:
 #if	defined(HASNULLFS)
 /*
  * If there is a saved nullfs vfs pointer, propagate its device number.
- * 
  */
 	if (nvfs) {
+
+# if	defined(HASSTATVFS)
+	    dev = nvfs->fsid.__fsid_val[0];
+# else	/* !defined(HASSTATVFS) */
 	    dev = nvfs->fsid.val[0];
+# endif	/* defined(HASSTATVFS) */
+
 	    devs = 1;
 	}
 #endif	/* defined(HASNULLFS) */
