@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.45 2003/06/11 11:43:07 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.48 2004/03/10 23:53:41 abe Exp $";
 #endif
 
 
@@ -67,6 +67,7 @@ _PROTOTYPE(static char isvlocked,(struct vnode *va));
 _PROTOTYPE(static int readinode,(KA_T ia, struct inode *i));
 _PROTOTYPE(static void read_mi,(KA_T s, dev_t *dev, caddr_t so, int *so_st, KA_T *so_ad, struct l_dev **sdp));
 
+
 #if	solaris>=20500
 # if	solaris>=20600
 _PROTOTYPE(static int read_nan,(KA_T na, KA_T aa, struct fnnode *rn));
@@ -101,7 +102,13 @@ _PROTOTYPE(static int read_cni,(struct snode *s, struct vnode *rv,
 _PROTOTYPE(static int read_ncn,(KA_T na, KA_T ca, struct cnode *cn));
 _PROTOTYPE(static int read_nln,(KA_T na, KA_T la, struct lnode *ln));
 _PROTOTYPE(static int read_nnn,(KA_T na, KA_T nna, struct namenode *n));
+
+#if	solaris<100000
 _PROTOTYPE(static void savesockmod,(struct so_so *so, struct so_so *sop, int *so_st));
+#else	/* solaris>=100000 */
+_PROTOTYPE(static int read_ndvn,(KA_T na, KA_T da, struct dv_node *dv,
+				 dev_t *dev, unsigned char *devs));
+#endif	/* solaris<100000 */
 
 
 /*
@@ -112,6 +119,7 @@ static KA_T Auvops;		/* auto_vnodeops address */
 static KA_T Avops;		/* afs_vnodeops address */
 static KA_T Cvops;		/* cache_vnodeops address */
 static KA_T Dvops;		/* door_vnodeops address */
+static KA_T Devops;		/* dv_vnodeops address */
 static KA_T Fdops;		/* fdvnodeops address */
 static KA_T Fvops;		/* fifo_vnodeops address */
 static KA_T Hvops;		/* hsfs_vnodeops address */
@@ -119,13 +127,17 @@ static KA_T Lvops;		/* lo_vnodeops address */
 static KA_T Mntops;		/* mntvnodeops address */
 static KA_T Mvops;		/* mvfs_vnodeops address */
 static KA_T N3vops;		/* nfs3_vnodeops address */
+static KA_T N4vops;		/* nfs4_vnodeops address */
 static KA_T Nmvops;		/* nm_vnodeops address */
 static KA_T Nvops;		/* nfs_vnodeops address */
 static KA_T Pdvops;		/* pcfs_dvnodeops address */
 static KA_T Pfvops;		/* pcfs_fvnodeops address */
 static KA_T Prvops;		/* prvnodeops address */
-static KA_T Samvops;		/* samfs_vnodeops address */
+static KA_T Sam1vops;		/* samfs_vnodeops address */
+static KA_T Sam2vops;		/* samfs_client_vnodeops address */
 static KA_T Sckvops;		/* sock_vnodeops address */
+static KA_T Sncavops;		/* socknca_vnodeops address */
+static KA_T Stpivops;		/* socktpi_vnodeops address */
 static KA_T Spvops;		/* spec_vnodeops address */
 static KA_T Tvops;		/* tmp_vnodeops address */
 static KA_T Uvops;		/* ufs_vnodeops address */
@@ -134,6 +146,24 @@ static KA_T Vvops[VXVOP_NUM];	/* addresses of:
 				 *   fdd_vnops (VXVOP_FDD), and
 				 *   vx_vnodeops (VXVOP_REG) */
 
+/*
+ * Local macros
+ *
+ * GETVOPS() -- get direct or indirect *vnodeops address
+ */
+
+#if	defined(VOPNAME_OPEN) && solaris>=100000
+#define	GETVOPS(name, nl, ops) \
+    if (get_Nl_value(name, nl, &ops) < 0) \
+	ops = (KA_T)0; \
+    else if (kread(ops, (char *)&ops, sizeof(ops))) \
+	ops = (KA_T)0
+#else	/* !defined(VOPNAME_OPEN) || solaris<100000 */
+#define	GETVOPS(name, nl, ops) \
+    if (get_Nl_value(name, nl, &ops) < 0) \
+	ops = (KA_T)0
+#endif	/* defined(VOPNAME_OPEN) && solaris>=100000 */
+
 
 /*
  * ent_fa() - enter fattach addresses in NAME column addition
@@ -296,7 +326,7 @@ isvlocked(va)
 	KA_T fp;
 #endif	/* solaris<20500 */
 
-	int l;
+	int i, l;
 
 #if	solaris>=20300
 	struct lock_descriptor ld;
@@ -320,7 +350,7 @@ isvlocked(va)
 #endif	/* solaris>=20300 */
 
 	if (va->v_filocks == NULL)
-		return(' ');
+	    return(' ');
 
 #if	solaris<20500
 # if	solaris>20300 || (solaris==20300 && defined(P101318) && P101318>=45)
@@ -329,9 +359,11 @@ isvlocked(va)
 
 	{
 	    ff = fp = (KA_T)va->v_filocks;
+	    i = 0;
 	    do {
 		if (kread(fp, (char *)&f, sizeof(f)))
 		    return(' ');
+		i++;
 		if (f.set.l_pid != (pid_t)Lp->pid)
 		    continue;
 		if (f.set.l_whence == 0 && f.set.l_start == 0
@@ -349,15 +381,17 @@ isvlocked(va)
 		default:
 		    return('N');
 		}
-	    } while ((fp = (KA_T)f.next) && fp != ff);
+	    } while ((fp = (KA_T)f.next) && (fp != ff) && (i < 10000));
 	}
 #endif	/* solaris<20500 */
 
 #if	solaris>=20300
 	lf = lp = (KA_T)va->v_filocks;
+	i = 0;
 	do {
 	    if (kread(lp, (char *)&ld, sizeof(ld)))
 		return(' ');
+	    i++;
 	    if (!(LOCK_FLAGS & ACTIVE_LOCK) || LOCK_OWNER != (pid_t)Lp->pid)
 		continue;
 	    if (LOCK_START == 0
@@ -383,7 +417,7 @@ isvlocked(va)
 	    default:
 		return('L');
 	    }
-	} while ((lp = (KA_T)LOCK_NEXT) && lp != lf);
+	} while ((lp = (KA_T)LOCK_NEXT) && (lp != lf) && (i < 10000));
 	return(' ');
 #endif	/* solaris>=20300 */
 
@@ -533,7 +567,7 @@ process_node(va)
 	struct inode i;
 	struct lnode lo;
 	struct vfs kv;
-	int len, nl, snl, sepl;
+	int len, llc, nl, snl, sepl;
 	struct mvfsnode m;
 	struct mounts *mp;
 	struct namenode nn;
@@ -545,9 +579,6 @@ process_node(va)
 	struct snode rs;
 	struct snode s;
 	struct l_dev *sdp = (struct l_dev *)NULL;
-	KA_T so_ad[2];
-	struct so_so soso;
-	int so_st = 0;
 	size_t sz;
 	struct tmpnode t;
 	char tbuf[128], *ty, ubuf[128];
@@ -577,6 +608,18 @@ process_node(va)
 	int dns = 0;
 #endif	/* solaris >=20500 */
 
+#if	solaris<100000
+	KA_T so_ad[2];
+	struct so_so soso;
+	int so_st = 0;
+#else	/* solaris>=100000 */
+	dev_t dv_dev;
+	struct dv_node dv;
+	unsigned char dv_devs = 0;
+	unsigned char dvs = 0;
+#endif	/* solaris<100000 */
+
+
 #if	defined(HASPROCFS)
 	struct procfsid *pfi;
 	struct pid pids;
@@ -593,64 +636,46 @@ process_node(va)
 /*
  * Do first-time only operations.
  */
+
+#if	solaris<100000
 	so_ad[0] = so_ad[1] = (KA_T)0;
+#endif	/* solaris<100000 */
+
 	if (ft) {
-	    if (get_Nl_value("auvops", Drive_Nl, &Auvops) < 0)
-		Auvops = (KA_T)0;
-	    if (get_Nl_value("avops", Drive_Nl, &Avops) < 0 || !Avops) {
-		if (get_Nl_value("Avops", Drive_Nl, &Avops) < 0)
-		    Avops = (KA_T)0;
-	    }
-	    if (get_Nl_value("cvops", Drive_Nl, &Cvops) < 0)
-		Cvops = (KA_T)0;
-	    if (get_Nl_value("dvops", Drive_Nl, &Dvops) < 0)
-		Dvops = (KA_T)0;
-	    if (get_Nl_value("fdops", Drive_Nl, &Fdops) < 0)
-		Fdops = (KA_T)0;
-	    if (get_Nl_value("fvops", Drive_Nl, &Fvops) < 0)
-		Fvops = (KA_T)0;
-	    if (get_Nl_value("hvops", Drive_Nl, &Hvops) < 0)
-		Hvops = (KA_T)0;
-	    if (get_Nl_value("lvops", Drive_Nl, &Lvops) < 0)
-		Lvops = (KA_T)0;
-	    if (get_Nl_value("mntops", Drive_Nl, &Mntops) < 0)
-		Mntops = (KA_T)0;
-	    if (get_Nl_value("mvops", Drive_Nl, &Mvops) < 0)
-		Mvops = (KA_T)0;
-	    if (get_Nl_value("n3vops", Drive_Nl, &N3vops) < 0)
-		N3vops = (KA_T)0;
-	    if (get_Nl_value("nmvops", Drive_Nl, &Nmvops) < 0)
-		Nmvops = (KA_T)0;
-	    if (get_Nl_value("nvops", Drive_Nl, &Nvops) < 0)
-		Nvops = (KA_T)0;
-	    if (get_Nl_value("pdvops", Drive_Nl, &Pdvops) < 0)
-		Pdvops = (KA_T)0;
-	    if (get_Nl_value("pfvops", Drive_Nl, &Pfvops) < 0)
-		Pfvops = (KA_T)0;
-	    if (get_Nl_value("prvops", Drive_Nl, &Prvops) < 0)
-		Prvops = (KA_T)0;
-	    if (get_Nl_value("samvops", Drive_Nl, &Samvops) < 0)
-		Samvops = (KA_T)0;
-	    if (get_Nl_value("sckvops", Drive_Nl, &Sckvops) < 0)
-		Sckvops = (KA_T)0;
-	    if (get_Nl_value("spvops", Drive_Nl, &Spvops) < 0)
-		Spvops = (KA_T)0;
-	    if (get_Nl_value("tvops", Drive_Nl, &Tvops) < 0)
-		Tvops = (KA_T)0;
-	    if (get_Nl_value("uvops", Drive_Nl, &Uvops) < 0)
-		Uvops = (KA_T)0;
-	    if (VXVOP_FDD < VXVOP_NUM) {
-		if (get_Nl_value("vvfops", Drive_Nl, &Vvops[VXVOP_FDD]) < 0)
-		    Vvops[VXVOP_FDD] = (KA_T)0;
-	    }
-	    if (VXVOP_FDDCH < VXVOP_NUM) {
-		if (get_Nl_value("vvfcops", Drive_Nl, &Vvops[VXVOP_FDDCH]) < 0)
-		    Vvops[VXVOP_FDDCH] = (KA_T)0;
-	    }
-	    if (VXVOP_REG < VXVOP_NUM) {
-		if (get_Nl_value("vvops", Drive_Nl, &Vvops[VXVOP_REG]) < 0)
-		    Vvops[VXVOP_REG] = (KA_T)0;
-	    }
+	    GETVOPS("auvops", Drive_Nl, Auvops);
+	    GETVOPS("avops", Drive_Nl, Avops);
+	    if (!Avops)
+		GETVOPS("Avops", Drive_Nl, Avops);
+	    GETVOPS("cvops", Drive_Nl, Cvops);
+	    GETVOPS("devops", Drive_Nl, Devops);
+	    GETVOPS("dvops", Drive_Nl, Dvops);
+	    GETVOPS("fdops", Drive_Nl, Fdops);
+	    GETVOPS("fvops", Drive_Nl, Fvops);
+	    GETVOPS("hvops", Drive_Nl, Hvops);
+	    GETVOPS("lvops", Drive_Nl, Lvops);
+	    GETVOPS("mntops", Drive_Nl, Mntops);
+	    GETVOPS("mvops", Drive_Nl, Mvops);
+	    GETVOPS("n3vops", Drive_Nl, N3vops);
+	    GETVOPS("n4vops", Drive_Nl, N4vops);
+	    GETVOPS("nmvops", Drive_Nl, Nmvops);
+	    GETVOPS("nvops", Drive_Nl, Nvops);
+	    GETVOPS("pdvops", Drive_Nl, Pdvops);
+	    GETVOPS("pfvops", Drive_Nl, Pfvops);
+	    GETVOPS("prvops", Drive_Nl, Prvops);
+	    GETVOPS("sam1vops", Drive_Nl, Sam1vops);
+	    GETVOPS("sam2vops", Drive_Nl, Sam2vops);
+	    GETVOPS("sckvops", Drive_Nl, Sckvops);
+	    GETVOPS("sncavops", Drive_Nl, Sncavops);
+	    GETVOPS("stpivops", Drive_Nl, Stpivops);
+	    GETVOPS("spvops", Drive_Nl, Spvops);
+	    GETVOPS("tvops", Drive_Nl, Tvops);
+	    GETVOPS("uvops", Drive_Nl, Uvops);
+	    if (VXVOP_FDD < VXVOP_NUM)
+		GETVOPS("vvfops", Drive_Nl, Vvops[VXVOP_FDD]);
+	    if (VXVOP_FDDCH < VXVOP_NUM)
+		GETVOPS("vvfcops", Drive_Nl, Vvops[VXVOP_FDDCH]);
+	    if (VXVOP_REG < VXVOP_NUM)
+		GETVOPS("vvops", Drive_Nl, Vvops[VXVOP_REG]);
 	    ft = 0;
 	}
 /*
@@ -806,7 +831,6 @@ vfs_read_error:
  */
 	if (Spvops && Spvops == (KA_T)v->v_op) 
 	    Ntype = N_SPEC;
-
 /*
  * Determine the Solaris lock state.
  */
@@ -854,8 +878,14 @@ vfs_read_error:
 		if ((vty == N_STREAM || vty == N_REGLR) && vs) {
 		    Lf->is_stream = 1;
 		    vty = N_STREAM;
+
+#if	solaris<100000
 		    read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st,
 			    so_ad, &sdp);
+#else	/* solaris>=100000 */
+		    read_mi(vs, (dev_t *)&s.s_dev, NULL, NULL, NULL, &sdp);
+#endif	/* solaris<100000 */
+
 		    vs = (KA_T)NULL;
 		}
 	    }
@@ -879,6 +909,15 @@ vfs_read_error:
 
 		return;
 	    break;
+
+# if	solaris>=100000
+	case N_DEV:
+	    if (read_ndvn(va, (KA_T)v->v_data, &dv, &dv_dev, &dv_devs))
+		return;
+	    dvs = 1;
+	    break;
+# endif	/* solaris>=100000 */
+
 	case N_DOOR:
 	    if (read_ndn(va, (KA_T)v->v_data, &dn))
 		return;
@@ -994,17 +1033,30 @@ vfs_read_error:
 		return;
 	    break;
 	case N_LOFS:
+	    llc = rvs = 0;
 	    do {
-		if (read_nln(va, (KA_T)v->v_data, &lo))
+		if (read_nln(va,
+			     llc ? (KA_T)rv.v_data : (KA_T)v->v_data,
+			     &lo))
+		{
 		    return;
+		}
 		if (!(realvp = (KA_T)lo.lo_vp)) {
 		    (void) snpf(Namech, Namechl, "lnode at %s: no real vnode",
 			print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		    enter_nm(Namech);
 		    return;
 		}
-		if (read_nvn((KA_T)v->v_data, (KA_T)realvp, v))
+		if (read_nvn((KA_T)v->v_data, (KA_T)realvp, &rv))
 		    return;
-		vty_tmp = vop2ty(v);
+		rvs = 1;
+		llc++;
+		if (((vty_tmp = vop2ty(&rv)) == N_LOFS) && (llc > 1000)) {
+		    (void) snpf(Namech, Namechl, "lnode at %s: loop > 1000",
+			print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		    enter_nm(Namech);
+		    return;
+		}
 	    } while (vty_tmp == N_LOFS);
 	    break;
 	case N_PCFS:
@@ -1031,8 +1083,14 @@ vfs_read_error:
 		return;
 	    if (vs) {
 		Lf->is_stream = 1;
+
+#if	solaris<100000
 		read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st, so_ad,
 		    &sdp);
+#else	/* solaris>=100000 */
+		read_mi(vs, (dev_t *)&s.s_dev, NULL, NULL, NULL, &sdp);
+#endif	/* solaris<100000 */
+
 		vs = (KA_T)NULL;
 	    }
 	    break;
@@ -1070,8 +1128,14 @@ vfs_read_error:
 	 */
 	    if (vty == N_STREAM && vs) {
 		Lf->is_stream = 1;
+
+#if	solaris<100000
 		read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st, so_ad,
 		    &sdp);
+#else	/* solaris>=100000 */
+		read_mi(vs, (dev_t *)&s.s_dev, NULL, NULL, NULL, &sdp);
+#endif	/* solaris<100000 */
+
 		vs = (KA_T)NULL;
 	    }
 	/*
@@ -1081,11 +1145,17 @@ vfs_read_error:
 		if (Ntype != N_FIFO && vs)
 		    vty = N_STREAM;
 		else {
+
+#if	solaris<100000
 		    (void) snpf(Namech, Namechl,
 			"unknown file system type, v_op: %s",
 			print_kptr((KA_T)v->v_op, (char *)NULL, 0));
-		    enter_nm(Namech);
-		    return;
+#else	/* solaris>=100000 */
+		    (void) snpf(Namech, Namechl,
+			"unknown file system type: %s",
+			fxs ? Fsinfo[fx] : "(unknown)");
+#endif	/* solaris<100000 */
+
 		}
 	    }
 	    if (Ntype == N_NM || Ntype == N_AFS)
@@ -1093,6 +1163,7 @@ vfs_read_error:
 	/*
 	 * Base further processing on the "real" vnode.
 	 */
+	    Lf->lock = isvlocked(v);
 	    switch (vty) {
 
 #if	defined(HAS_AFS)
@@ -1113,6 +1184,15 @@ vfs_read_error:
 
 		    return;
 		break;
+
+# if	solaris>=100000
+	    case N_DEV:
+		if (read_ndvn(va, (KA_T)v->v_data, &dv, &dv_dev, &dv_devs))
+		    return;
+		dvs = 1;
+		break;
+# endif	/* solaris>=100000 */
+
 	    case N_DOOR:
 
 # if	solaris<20600
@@ -1168,8 +1248,14 @@ vfs_read_error:
 	    case N_STREAM:
 		if (vs) {
 		    Lf->is_stream = 1;
-		    read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st,
-			    so_ad, &sdp);
+
+#if	solaris<100000
+		read_mi(vs, (dev_t *)&s.s_dev, (caddr_t)&soso, &so_st, so_ad,
+		    &sdp);
+#else	/* solaris>=100000 */
+		read_mi(vs, (dev_t *)&s.s_dev, NULL, NULL, NULL, &sdp);
+#endif	/* solaris<100000 */
+
 		    vs = (KA_T)NULL;
 		}
 		break;
@@ -1216,6 +1302,21 @@ vfs_read_error:
 		devs = 1;
 	    }
 	    break;
+
+# if	solaris>=100000
+	case N_DEV:
+	    if (dv_devs) {
+		dev = dv_dev;
+		devs = 1;
+	    } else if (vfs) {
+		dev = vfs->dev;
+		devs = 1;
+	    }
+	    rdev = v->v_rdev;
+	    rdevs = 1;
+	    break;
+# endif	/* solaris>=100000 */
+
 	case N_DOOR:
 
 # if	solaris<20600
@@ -1270,10 +1371,24 @@ vfs_read_error:
 	     *    Check for matches on sockaddr_un.sun_path names.
 	     */
 		if (!sdp)
-		    sdp = finddev(&DevDev, &so.so_vnode.v_rdev, LOOKDEV_ALL);
+		    sdp = finddev(&DevDev,
+
+#if	solaris<100000
+				  &so.so_vnode.v_rdev,
+#else	/* solaris>=100000 */
+				  &so.so_dev,
+#endif	/* solaris<100000 */
+
+				  LOOKDEV_ALL);
 		if (sdp) {
 		    dev = DevDev;
+
+#if	solaris<100000
 		    rdev = so.so_vnode.v_rdev;
+#else	/* solaris>=100000 */
+		    rdev = so.so_dev;
+#endif	/* solaris<100000 */
+
 		    trdev = sdp->rdev;
 		    devs = rdevs = trdevs = 1;
 		    Lf->inode = (unsigned long)sdp->inode;
@@ -1437,6 +1552,8 @@ vfs_read_error:
 	    }
 	    break;
 	case N_SPEC:
+
+#if	solaris<100000
 	    if (((Ntype = vty) == N_STREAM) && so_st) {
 		if (Funix)
 		    Lf->sf |= SELUNX;
@@ -1510,7 +1627,10 @@ vfs_read_error:
 			}
 		    }
 		}
-	    } else {
+	    } else
+#endif	/* solaris<100000 */
+
+	    {
 		if (vfs) {
 		    dev = vfs->dev;
 		    devs = 1;
@@ -1594,6 +1714,16 @@ vfs_read_error:
 
 	    Lf->inp_ty = 1;
 	    break;
+
+# if	solaris>=100000
+	case N_DEV:
+	    if (dvs) {
+		Lf->inode = (unsigned long)dv.dv_ino;
+		Lf->inp_ty = 1;
+	    }
+	    break;
+# endif	/* solaris>=100000 */
+
 	case N_DOOR:
 	    if (nns && (Lf->inode = (unsigned long)nn.nm_vattr.va_nodeid)) {
 		Lf->inp_ty = 1;
@@ -1618,13 +1748,7 @@ vfs_read_error:
 	    Lf->inp_ty = 1;
 	    break;
 	case N_HSFS:
-
-#if	defined(HAS_HS_NODEID)
 	    Lf->inode = (unsigned long)h.hs_nodeid;
-#else	/* defined(HAS_HS_NODEID) */
-	    Lf->inode = (unsigned long)h.hs_dirent.ext_lbn;
-#endif	/* defined(HAS_HS_NODEID) */
-
 	    Lf->inp_ty = 1;
 	    break;
 
@@ -1697,6 +1821,8 @@ vfs_read_error:
 	case N_SAMFS:
 	    break;		/* No more SAM-FS information is available. */
 	case N_STREAM:
+
+#if	solaris<100000
 	    if (so_st && soso.lux_dev.addr.tu_addr.ino) {
 		if (Lf->inp_ty) {
 		    nl = Lf->nma ? (int)strlen(Lf->nma) : 0;
@@ -1715,6 +1841,8 @@ vfs_read_error:
 		    Lf->inp_ty = 1;
 		}
 	    }
+#endif	/* solaris<100000 */
+
 	    break;
 	case N_TMP:
 	    Lf->inode = (unsigned long)t.tn_attr.va_nodeid;
@@ -1786,6 +1914,14 @@ vfs_read_error:
 		Lf->sz = (SZOFFTYPE)nn.nm_vattr.va_size;
 		Lf->sz_def = 1;
 		break;
+
+# if	solaris>=100000
+	     case N_DEV:
+		if (!Fsize)
+		    Lf->off_def = 1;
+		break;
+# endif	/* solaris>=100000 */
+
 	    case N_DOOR:
 	    case N_FIFO:
 		if (!Fsize)
@@ -1902,6 +2038,16 @@ vfs_read_error:
 		Lf->nlink = (long)nn.nm_vattr.va_nlink;
 		Lf->nlink_def = 1;
 		break;
+
+# if	solaris>=100000
+	    case N_DEV:
+		if (dvs) {
+		    Lf->nlink = (long)dv.dv_nlink;
+		    Lf->nlink_def = 1;
+		}
+		break;
+# endif	/* solaris>=100000 */
+
 	    case N_DOOR:
 		Lf->nlink = (long)v->v_count;
 		Lf->nlink_def = 1;
@@ -2030,11 +2176,22 @@ vfs_read_error:
 	if (vfs) {
 	    Lf->fsdir = vfs->dir;
 	    Lf->fsdev = vfs->fsname;
-	    if (!Lf->fsdir && !Lf->fsdev && kvs && fxs)
+	    if (!Lf->fsdir && !Lf->fsdev && kvs && fxs) {
+
+	    /*
+	     * The file system names are unknown.
+	     *
+	     * Set the file system device to the file system type and clear
+	     * the doubtful device numbers.
+	     */
 		Lf->fsdev = Fsinfo[fx];
+		devs = 0;
+		rdevs = 0;
+	    }
 
 #if	defined(HASFSINO)
-	    Lf->fs_ino = vfs->fs_ino;
+	    else
+		Lf->fs_ino = vfs->fs_ino;
 #endif	/* defined(HASFSINO) */
 
 	}
@@ -2493,7 +2650,7 @@ read_mi(s, rdev, so, so_st, so_ad, sdp)
 	mn[nl] = '\0';
 	qp = (KA_T)sd.sd_wrq;
 	for (i = 0; qp && i < 20; i++, qp = (KA_T)q.q_next) {
-	    if (!qp ||  kread(qp, (char *)&q, sizeof(q)))
+	    if (!qp || kread(qp, (char *)&q, sizeof(q)))
 		break;
 	    if ((ka = (KA_T)q.q_qinfo) == (KA_T)NULL
 	    ||  kread(ka, (char *)&qi, sizeof(qi)))
@@ -2508,6 +2665,8 @@ read_mi(s, rdev, so, so_st, so_ad, sdp)
 		continue;
 	    if (j >= 4 && strcmp(&mn[j - 4], "head") == 0)
 		continue;
+
+#if	solaris<100000
 	    if (strcmp(mn, "sockmod") == 0) {
 
 	    /*
@@ -2526,6 +2685,8 @@ read_mi(s, rdev, so, so_st, so_ad, sdp)
 		    }
 		}
 	    }
+#endif	/* solaris<100000 */
+
 	    if (k) {
 		if ((k + 2) > (Namechl - 1))
 		    break;
@@ -3258,6 +3419,7 @@ read_nvn(na, va, v)
 }
 
 
+#if	solaris<100000
 /*
  * savesockmod() - save addresses from sockmod so_so structure
  */
@@ -3415,6 +3577,8 @@ savesockmod(so, sop, so_st)
 #endif	/* solaris<20500 */
 
 }
+#endif	/* solaris<100000 */
+
 
 
 /*
@@ -3427,11 +3591,11 @@ vop2ty(vp)
 {
 	register int i;
 
-#if	defined(HAS_AFS)
+# if	defined(HAS_AFS)
 	int afs = 0;			/* afs test status: -1 = no AFS
 					 *		     0 = not tested
 					 *		     1 = AFS */
-#endif	/* defined(HAS_AFS) */
+# endif	/* defined(HAS_AFS) */
 
 	if (!vp->v_op)
 		return(-1);
@@ -3443,6 +3607,8 @@ vop2ty(vp)
 	    return(N_NFS);
 	else if (N3vops && N3vops == (KA_T)vp->v_op)
 	    return(N_NFS);
+	else if (N4vops && N4vops == (KA_T)vp->v_op)
+	    return(N_NFS);
 
 # if	defined(HASVXFS)
 	for (i = 0; i < VXVOP_NUM; i++) {
@@ -3470,6 +3636,8 @@ vop2ty(vp)
 	    return(N_CACHE);
 	else if (Dvops && Dvops == (KA_T)vp->v_op)
 	    return(N_DOOR);
+	else if (Devops && Devops == (KA_T)vp->v_op)
+	    return(N_DEV);
 	else if (Fdops  && Fdops  == (KA_T)vp->v_op)
 	    return(N_FD);
 	else if (Fvops && Fvops == (KA_T)vp->v_op)
@@ -3480,12 +3648,18 @@ vop2ty(vp)
 	    return(N_NM);
 	else if (Prvops && Prvops == (KA_T)vp->v_op)
 	    return(N_PROC);
-	else if (Samvops && Samvops == (KA_T)vp->v_op)
+	else if (Sam1vops && Sam1vops == (KA_T)vp->v_op)
 	    return(N_SAMFS);
-	else if (Sckvops && Sckvops == (KA_T)vp->v_op)
+	else if (Sam2vops && Sam2vops == (KA_T)vp->v_op)
+	    return(N_SAMFS);
+	else if ((Sckvops  && Sckvops  == (KA_T)vp->v_op)
+	     ||  (Sncavops && Sncavops == (KA_T)vp->v_op)
+	     ||  (Stpivops && Stpivops == (KA_T)vp->v_op)
+	) {
 	    return(N_SOCK);
+	}
 
-#if	defined(HAS_AFS)
+# if	defined(HAS_AFS)
 /*
  * Caution: this should be the last test in vop2ty().
  */
@@ -3513,8 +3687,73 @@ vop2ty(vp)
 	}
 	return(-1);
 
-#else	/* !defined(HAS_AFS) */
+# else	/* !defined(HAS_AFS) */
 	return(-1);
-#endif	/* defined(HAS_AFS) */
+# endif	/* defined(HAS_AFS) */
 
 }
+
+
+#if	solaris>=100000
+/*
+ * read_ndvn() -- read node's dv_node
+ */
+
+static int
+read_ndvn(na, da, dv, dev, devs)
+	KA_T na;			/* containing vnode's address */
+	KA_T da;			/* containing vnode's v_data */
+	struct dv_node *dv;		/* dv_node receiver */
+	dev_t *dev;			/* underlying file system device
+					 * number receptor */
+	unsigned char *devs;		/* status of *dev */
+{
+	struct vnode rv;
+	struct snode s;
+	char tbuf[32];
+	struct vfs v;
+/*
+ * Read the snode.
+ */
+	if (!da || kread((KA_T)da, (char *)&s, sizeof(s))) {
+	    (void) snpf(Namech, Namechl,
+		"dv_node vnode at %s: can't read snode: %s",
+		print_kptr(na, tbuf, sizeof(tbuf)),
+		print_kptr(da, (char *)NULL, 0));
+	    enter_nm(Namech);
+	    return(1);
+	}
+/*
+ * Read the snode's real vnode.
+ */
+	if (!s.s_realvp
+	|| kread((KA_T)s.s_realvp, (char *)&rv, sizeof(struct dv_node)))
+	{
+	    (void) snpf(Namech, Namechl,
+		"dv_node snode at %s: can't read real vnode: %s",
+		print_kptr(da, tbuf, sizeof(tbuf)),
+		print_kptr((KA_T)s.s_realvp, (char *)NULL, 0));
+	    enter_nm(Namech);
+	    return(1);
+	}
+/*
+ * Read the real vnode's dv_node.
+ */
+	if (!rv.v_data || kread((KA_T)rv.v_data, (char *)dv, sizeof(rv))) {
+	    (void) snpf(Namech, Namechl,
+		"dv_node real vnode at %s: can't read dv_node: %s",
+		print_kptr((KA_T)s.s_realvp, tbuf, sizeof(tbuf)),
+		print_kptr((KA_T)rv.v_data, (char *)NULL, 0));
+	    enter_nm(Namech);
+	    return(1);
+	}
+/*
+ * Return the device number of the underlying file system, if possible.
+ */
+	if (rv.v_vfsp && !kread((KA_T)rv.v_vfsp, (char *)&v, sizeof(v))) {
+	    *dev = v.vfs_dev;
+	    *devs = 1;
+	}
+	return(0);
+}
+#endif	/* solaris<100000 */
