@@ -1,5 +1,5 @@
 /*
- * dproc.c - Solaris lsof functions for accessing process infomation
+ * dproc.c - Solaris lsof functions for accessing process information
  */
 
 
@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.25 2002/10/08 20:17:30 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.28 2004/03/10 23:53:41 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -41,6 +41,10 @@ static char *rcsid = "$Id: dproc.c,v 1.25 2002/10/08 20:17:30 abe Exp $";
 #include "kernelbase.h"
 #endif	/* solaris<20500 */
 
+#if	defined(HAS_CRED_IMPL_H)
+#include <sys/cred_impl.h>
+#endif	/* defined(HAS_CRED_IMPL_H) */
+
 
 /*
  * Local definitions
@@ -103,17 +107,25 @@ static int Kmd = -1;			/* memory device file descriptor */
 static KA_T Kb = (KA_T)NULL;		/* KERNELBASE for Solaris 2.5 */
 #endif	/* solaris>=20500 */
 
-static int Knp = PROCDFLT;		/* numer of proc structures known
-					 * to the kernel */
-static int Np;				/* number of P[], Pgid[], and Pid[]
+static int Np;				/* number of P[], Pgid[] and Pid[]
 					 * entries  */
+static int Npa = 0;			/* number of P[], Pgid[] and Pid[]
+					 * entries for which space has been
+					 * allocated */
+static KA_T Nproc = 0;			/* kernel's nproc address */
 static struct proc *P = NULL;		/* local proc structure table */
-static KA_T Sgvops;			/* [_]segvn_ops address */
 static int *Pgid = NULL;		/* process group IDs for P[] entries */
 static int *Pid = NULL;			/* PIDs for P[] entries */
+static KA_T PrAct = (KA_T)NULL;		/* kernel's *practive address */
 static gid_t Savedgid;			/* saved (effective) GID */
+static KA_T Sgvops;			/* [_]segvn_ops address */
 static int Switchgid = 0;		/* must switch GIDs for kvm_open() */
 
+#if	defined(HASZONES)
+static znhash_t **ZoneNm = (znhash_t **)NULL;
+					/* zone names hash buckets */
+#endif	/* defined(HASZONES) */
+
 
 /*
  * Local function prototypes
@@ -132,6 +144,10 @@ _PROTOTYPE(static void readkam,(KA_T addr));
 _PROTOTYPE(extern u_longlong_t kvm_physaddr,(kvm_t *, struct as *, u_int));
 #endif	/* solaris>=20501 && solaris<70000 */
 
+#if	defined(HASZONES)
+_PROTOTYPE(static int hash_zn,(char *zn));
+#endif	/* defined(HASZONES) */
+
 
 
 /*
@@ -202,6 +218,13 @@ gather_proc_info()
 	static char *pofb = (char *)NULL;
 #endif	/* defined(HASFSTRUCT) */
 
+#if	defined(HASZONES)
+	struct zone z;
+	int zh;
+	char zn[ZONENAME_MAX + 1];
+	znhash_t *zp, *zpn;
+#endif	/* defined(HASZONES) */
+
 /*
  * Do first-time only operations.
  */
@@ -215,24 +238,24 @@ gather_proc_info()
 	else
 
 	/*
-	 * If not the first time, re-read the kernel's address space map.
+	 * If not the first time and the ALLKMEM device isn't available,
+	 * re-read the kernel's address space map.
 	 */
-	    readkam((KA_T)NULL);
+	    if (!HasALLKMEM)
+		readkam((KA_T)NULL);
 #endif	/* solaris>=20501 */
 
 /*
- * Read the process table.
+ * Read the linked-list process table.
  *
- * The Solaris process table is a linked list whose head pointer is acquired
- * by open_kvm()'s call to kvm_open().  For Solaris close_kvm() is called so
- * the process table head pointer can be re-acquired by a call to open_kvm()
- * immediately before the actual reading of the process table.
- *
- * Since the SunOS 4.1.x process table is sequential, there's no worry about
- * a stale or incorrect list head pointer.  Thus no close_kvm() is needed.
- * A kvm_setproc() call is sufficient.
+ * If the system doesn't have the ALLKMEM device, it is necessary to use the
+ * KVM library to read the process table.  Since kvm_open() acquires the head
+ * of the linked list, it is necssary to call close_kvm() first, so that the
+ * read_proc() function's open_kvm() call will cause kvm_open() to acquire a
+ * new active chain head address.
  */
-	close_kvm();
+	if (!HasALLKMEM)
+	    close_kvm();
 	read_proc();
 /*
  * Loop through processes.
@@ -265,6 +288,23 @@ gather_proc_info()
 	 */
 	    if  (is_proc_excl(pid, pgid, (UID_ARG)uid, &pss, &sf))
 		continue;
+
+#if	defined(HASZONES)
+	/*
+	 * If the -z (zone) option was specified, get the zone name.
+	 */
+	    if (Fzone) {
+		zn[0] = zn[sizeof(zn) - 1] = '\0';
+		if (p->p_zone
+		&& !kread((KA_T)p->p_zone, (char *)&z, sizeof(z)))
+		{
+		    if (!z.zone_name
+		    ||  kread((KA_T)z.zone_name, (char *)&zn, sizeof(zn) - 1))
+			zn[0] = '\0';
+		}
+	    }
+#endif	/* defined(HASZONES) */
+
 	/*
 	 * Get the user area associated with the process.
 	 */
@@ -277,6 +317,73 @@ gather_proc_info()
 	    alloc_lproc(pid, pgid, (int)p->p_ppid, (UID_ARG)uid, u->u_comm,
 		(int)pss, (int)sf);
 	    Plf = (struct lfile *)NULL;
+
+#if	defined(HASZONES)
+	/*
+	 * If zone processing is enabled and requested, and if there is a zone
+	 * name:
+	 *
+	 *	o Skip processes excluded by zone name.
+	 *	o Save zone name.
+	 */
+	    if (Fzone && zn[0]) {
+		zh = hash_zn(zn);
+		if (ZoneArg) {
+
+		/*
+		 * See if zone name excludes the process.
+		 */
+		    for (zp = ZoneArg[zh]; zp; zp = zp->next) {
+			if (!strcmp(zn, zp->zn))
+			    break;
+		    }
+		    if (!zp)
+			continue;
+		    zp->f = 1;
+		    Lp->pss |= PS_PRI;
+		    Lp->sf |= SELZONE;
+		}
+	    /*
+	     * Make sure the zone name is cached, then save a pointer to it in
+	     * the local proc structure.
+	     */
+		if (!ZoneNm) {
+		    if (!(ZoneNm = (znhash_t **)calloc(HASHZONE,
+					        sizeof(znhash_t *))))
+		    {
+			(void) fprintf(stderr,
+			    "%s: no space for zone name hash\n", Pn);
+			Exit(1);
+		    }
+		}
+		for (zp = ZoneNm[zh]; zp; zp = zp->next) {
+		    if (!strcmp(zn, zp->zn))
+			break;
+		}
+		if (!zp) {
+
+		/*
+		 * The zone name isn't cached, so cache it.
+		 */
+		    if (!(zp = (znhash_t *)malloc((MALLOC_S)sizeof(znhash_t))))
+		    {
+			(void) fprintf(stderr,
+			    "%s: no zone name cache space: %s\n", Pn, zn);
+			Exit(1);
+		    }
+		    if (!(zp->zn = mkstrcpy(zn, (MALLOC_S *)NULL))) {
+			(void) fprintf(stderr,
+			    "%s: no zone name space at PID %d: %s\n",
+			    Pn, (int)Lp->pid, zn);
+			Exit(1);
+		    }
+		    zp->next = ZoneNm[zh];
+		    ZoneNm[zh] = zp;
+		}
+		Lp->zn = zp->zn;
+	    }
+#endif	/* defined(HASZONES) */
+
 	/*
 	 * Save file count.
 	 */
@@ -398,6 +505,7 @@ get_kernel_access()
 {
 	char er[64];
 	int i;
+	struct stat sb;
 	KA_T v;
 
 #if	defined(HAS_AFS)
@@ -411,7 +519,9 @@ get_kernel_access()
 
 #if	solaris>=70000
 /*
- * Compare the Solaris 7 lsof compilation bit size with the kernel bit size.
+ * Compare the Solaris 7 and above lsof compilation bit size with the kernel
+ * bit size.
+ *
  * Quit on a mismatch.
  */
 	{
@@ -497,13 +607,23 @@ get_kernel_access()
 	}
 #endif	/* defined(HAS_AFS) */
 
+/*
+ * Determine the availability of the ALLKMEM device.  If it is available, the
+ * active processes will be gathered directly from the active process chain.
+ *
+ * If ALLKMEM isn't available, the active processes will be gathered via the
+ * kvm_*proc() functions.
+ */
+	if (statsafely(ALLKMEM, &sb) == 0)
+	    HasALLKMEM = 1;
+
 #if	defined(WILLDROPGID)
 /*
- * If Solaris kernel memory is coming from KMEM, and the process is willing
- * to surrender GID permission, set up for GID switching after the first
- * call to open_kvm().
+ * If Solaris kernel memory is coming from KMEM, the process is willing to
+ * surrender GID permission, and the ALLKMEM device is not available, set up
+ * for GID switching after the first call to open_kvm().
  */
-	if (!Memory) {
+	if (!Memory && !HasALLKMEM) {
 	    Savedgid = getegid();
 	    if (Setgid)
 		Switchgid = 1;
@@ -526,13 +646,6 @@ get_kernel_access()
  * Open access to kernel memory.
  */
 	open_kvm();
-/*
- * Get a proc structure count estimate.
- */
-	if (get_Nl_value("nproc", Drive_Nl, &v) < 0 || !v
-	||  kread((KA_T)v, (char *)&Knp, sizeof(Knp))
-	||  Knp < 1)
-	    Knp = PROCDFLT;
 
 #if	solaris>=20500
 /*
@@ -555,24 +668,120 @@ get_kernel_access()
 	if (get_Nl_value("clmaj", Drive_Nl, &v) >= 0 && v
 	&&  kread((KA_T)v, (char *)&CloneMaj, sizeof(CloneMaj)) == 0)
 	    HaveCloneMaj = 1;
+/*
+ * If the ALLKMEM device is available, check for the address of the kernel's
+ * active process chain.  If it's not available, clear the ALLKMEM status.
+ */
+	if (HasALLKMEM) {
+	   if ((get_Nl_value("pract", Drive_Nl, &PrAct) < 0) || !PrAct)
+		HasALLKMEM = 0;
+	}
 
 #if	solaris>=20501
 /*
- * Get the kernel's virtual to physical map structure for Solaris 2.5.1 and
- * above.
+ * If the ALLKMEM device isn't available, get the kernel's virtual to physical
+ * map structure for Solaris 2.5.1 and above.
  */
-	if (get_Nl_value("kasp", Drive_Nl, &v) >= 0 && v) {
-	    PageSz = getpagesize();
-	    PSMask = PageSz - 1;
-	    for (i = 1, PSShft = 0; i < PageSz; i <<= 1, PSShft++)
-		;
-	    (void) readkam(v);
+	if (!HasALLKMEM) {
+	    if (get_Nl_value("kasp", Drive_Nl, &v) >= 0 && v) {
+		PageSz = getpagesize();
+		PSMask = PageSz - 1;
+		for (i = 1, PSShft = 0; i < PageSz; i <<= 1, PSShft++)
+		    ;
+		(void) readkam(v);
+	    }
 	}
 #endif	/* solaris>=20501 */
 
+#if	defined(WILLDROPGID)
+/*
+ * If the ALLKMEM device is available -- i.e., we're not using the kvm_*proc()
+ * functions to read proc structures -- and if we're willing to drop setgid
+ * permission, do so.
+ */
+	if (HasALLKMEM)
+	    (void) dropgid();
+#endif	/* defined(WILLDROPGID) */
+
 }
 
 
+#if	defined(HASZONES)
+/*
+ * enter_zone_arg() - enter zone name argument
+ */
+
+int
+enter_zone_arg(zn)
+	char *zn;				/* zone name */
+{
+	int zh;
+	znhash_t *zp, *zpn;
+/*
+ * Allocate zone argument hash space, as required.
+ */
+	if (!ZoneArg) {
+	    if (!(ZoneArg = (znhash_t **)calloc(HASHZONE, sizeof(znhash_t *))))
+	    {
+		(void) fprintf(stderr, "%s: no space for zone arg hash\n", Pn);
+		Exit(1);
+	    }
+	}
+/*
+ * Hash the zone name and search the argument hash.
+ */
+	zh = hash_zn(zn);
+	for (zp = ZoneArg[zh]; zp; zp = zp->next) {
+	    if (!strcmp(zp->zn, zn))
+		break;
+	}
+	if (zp)	{
+
+	/*
+	 * Process a duplicate.
+	 */
+	    if (!Fwarn)
+		(void) fprintf(stderr, "%s: duplicate zone name: %s\n", Pn, zn);
+	    return(1);
+	}
+/*
+ * Create a new hash entry and link it to its bucket.
+ */
+	if (!(zpn = (znhash_t *)malloc((MALLOC_S)sizeof(znhash_t)))) {
+	    (void) fprintf(stderr, "%s no hash space for zone: %s\n", Pn, zn);
+	    Exit(1);
+	}
+	zpn->f = 0;
+	zpn->zn = zn;
+	zpn->next = ZoneArg[zh];
+	ZoneArg[zh] = zpn;
+	return(0);
+}
+
+
+/*
+ * hash_zn() - hash zone name
+ */
+
+static int
+hash_zn(zn)
+	char *zn;				/* zone name */
+{
+	register int i, h;
+	size_t l;
+
+	if (!(l = strlen(zn)))
+	    return(0);
+	if (l == 1)
+	    return((int)*zn & (HASHZONE - 1));
+	for (i = h = 0; i < (int)(l - 1); i++) {
+	    h ^= ((int)zn[i] * (int)zn[i+1]) << ((i*3)%13);
+	}
+	return(h & (HASHZONE - 1));
+}
+#endif	/* defined(HASZONES) */
+
+
 /*
  * initialize() - perform all initialization
  */
@@ -630,13 +839,16 @@ kread(addr, buf, len)
 #if	solaris>=20501
 
 /*
+ * Do extra address checking for Solaris above 2.5 when the ALLKMEM device
+ * isn't available.
+ *
  * Make sure the virtual address represents real physical memory by testing
  * it with kvm_physaddr().
  *
  * For Solaris below 7 read the kernel data with llseek() and read().  For
  * Solaris 7 and above use kvm_pread().
  */
-	if (Kas) {
+	if (Kas && !HasALLKMEM) {
 
 # if	solaris>20501
 	    register int b2r;
@@ -1112,10 +1324,10 @@ readkam(addr)
 static void
 read_proc()
 {
+	int ct, ctl, knp, n, try;
 	MALLOC_S len;
-	static int sz = 0;
-	int try;
 	struct proc *p;
+	KA_T pa, paf, pan;
 	struct pid pg, pids;
 /*
  * Try PROCTRYLM times to read a valid proc table.
@@ -1123,44 +1335,149 @@ read_proc()
 	for (try = 0; try < PROCTRYLM; try++) {
 
 	/*
-	 * Pre-allocate proc structure space.
+	 * Get a proc structure count estimate.
 	 */
-	    if (sz == 0) {
-		sz = Knp + PROCDFLT/4;
-		len = (sz * sizeof(struct proc));
-		if (!(P = (struct proc *)malloc(len))) {
+	    if (get_Nl_value("nproc", Drive_Nl, &pa) < 0 || !pa
+	    ||  kread(pa, (char *)&knp, sizeof(knp))
+	    ||  knp < 1)
+		knp = PROCDFLT;
+	/*
+	 * Pre-allocate space, as required.
+	 */
+	    n = knp + PROCDFLT/4;
+	    if (n > Npa) {
+
+	    /*
+	     * Allocate proc structure space.
+	     */
+		len = (n * sizeof(struct proc));
+		if (P)
+		    P = (struct proc *)realloc((MALLOC_P *)P, len);
+		else
+		    P = (struct proc *)malloc(len);
+		if (!P) {
 		    (void) fprintf(stderr, "%s: no proc table space\n", Pn);
 		    Exit(1);
 		}
-	/*
-	 * Pre-allocate space for Solaris PGID and PID numbers.
-	 */
-		len = (MALLOC_S)(sz * sizeof(int));
+	    /*
+	     * Pre-allocate PGID and PID number space.
+	     */
+		len = (MALLOC_S)(n * sizeof(int));
 		if (Fpgid) {
-		    if (!(Pgid = (int *)malloc(len))) {
+		    if (Pgid)
+			Pgid = (int *)realloc((MALLOC_P *)Pgid, len);
+		    else
+			Pgid = (int *)malloc(len);
+		    if (!Pgid) {
 			(void) fprintf(stderr, "%s: no PGID table space\n", Pn);
 			Exit(1);
 		    }
 		}
-		if (!(Pid = (int *)malloc(len))) {
+		if (Pid)
+		    Pid = (int *)realloc((MALLOC_P *)Pid, len);
+		else
+		    Pid = (int *)malloc(len);
+		if (!Pid) {
 		    (void) fprintf(stderr, "%s: no PID table space\n", Pn);
 		    Exit(1);
 		}
+		Npa = n;
 	    }
-	/*
-	 * Prepare for a proc table scan.
-	 */
-	    open_kvm();
-	    if (kvm_setproc(Kd) != 0) {
-		(void) fprintf(stderr, "%s: kvm_setproc: %s\n", Pn,
-		    strerror(errno));
-		Exit(1);
+	    if (HasALLKMEM) {
+
+	    /*
+	     * Prepare for a proc table scan via direct reading of the active
+	     * chain.
+	     */
+		if (!PrAct || kread(PrAct, (char *)&paf, sizeof(pa))) {
+		    (void) fprintf(stderr, "%s: can't read practive from %s\n",
+			print_kptr(PrAct, (char *)NULL, 0));
+		    Exit(1);
+		}
+		ct = 1;
+		ctl = knp << 3;
+		pan = paf;
+		pa = (KA_T)NULL;
+	    } else {
+
+	    /*
+	     * Prepare for a proc table scan via the kvm_*proc() functions.
+	     */
+		open_kvm();
+		if (kvm_setproc(Kd) != 0) {
+		    (void) fprintf(stderr, "%s: kvm_setproc: %s\n", Pn,
+			strerror(errno));
+		    Exit(1);
+		}
 	    }
 	/*
 	 * Accumulate proc structures.
 	 */
 	    Np = 0;
-	    while ((p = kvm_nextproc(Kd))) {
+	    for (;;) {
+		if (Np >= Npa) {
+
+		/*
+		 * Expand the local proc table.
+		 */
+		    Npa += PROCDFLT/2;
+		    len = (MALLOC_S)(Npa * sizeof(struct proc));
+		    if (!(P = (struct proc *)realloc((MALLOC_P *)P, len))) {
+			(void) fprintf(stderr,
+			    "%s: no more (%d) proc space\n", Pn, Npa);
+			Exit(1);
+		    }
+		/*
+		 * Expand the PGID and PID tables.
+		 */
+		    len = (MALLOC_S)(Npa * sizeof(int));
+		    if (Fpgid) {
+			if (!(Pgid = (int *)realloc((MALLOC_P *)Pgid, len))) {
+			    (void) fprintf(stderr,
+				"%s: no more (%d) PGID space\n", Pn, Npa);
+			    Exit(1);
+			}
+		    }
+		    if (!(Pid = (int *)realloc((MALLOC_P *)Pid, len))) {
+			(void) fprintf(stderr,
+			    "%s: no more (%d) PID space\n", Pn, Npa);
+			Exit(1);
+		    }
+		}
+	    /*
+	     * Read the next proc structure.
+	     */
+		if (HasALLKMEM) {
+
+		/*
+		 * If the ALLKMEM device exists, read proc structures directly
+		 * from the active chain.
+		 */
+		    if (!pa)
+			pa = paf;
+		    else {
+			pa = pan;
+			if ((pan == paf) || (++ct > ctl))
+			   break;
+		    }
+		    if (!pa)
+			break;
+		    p = (struct proc *)&P[Np];
+		    if (kread(pa, (char *)p, sizeof(struct proc)))
+			break;
+		    pan = (KA_T)p->p_next;
+		} else {
+
+		/*
+		 * If the ALLKMEM device doesn't exist, read proc structures
+		 * via kbm_getproc().
+		 */
+		    if (!(p = kvm_nextproc(Kd)))
+			break;
+		}
+	    /*
+	     * Check process status.
+	     */
 		if (p->p_stat == 0 || p->p_stat == SZOMB)
 		    continue;
 
@@ -1183,53 +1500,27 @@ read_proc()
 		if (!p->p_pidp
 		||  kread((KA_T)p->p_pidp, (char *)&pids, sizeof(pids)))
 		    continue;
-		if (Np >= sz) {
-
-		/*
-		 * Expand the local proc table.
-		 */
-		    sz += PROCDFLT/2;
-		    len = (MALLOC_S)(sz * sizeof(struct proc));
-		    if (!(P = (struct proc *)realloc((MALLOC_P *)P, len))) {
-			(void) fprintf(stderr,
-			    "%s: no more (%d) proc space\n", Pn, sz);
-			Exit(1);
-		    }
-		/*
-		 * Expand the Solaris PGID and PID tables.
-		 */
-		    len = (MALLOC_S)(sz * sizeof(int));
-		    if (Fpgid) {
-			if (!(Pgid = (int *)realloc((MALLOC_P *)Pgid, len))) {
-			    (void) fprintf(stderr,
-				"%s: no more (%d) PGID space\n", Pn, sz);
-			    Exit(1);
-			}
-		    }
-		    if (!(Pid = (int *)realloc((MALLOC_P *)Pid, len))) {
-			(void) fprintf(stderr,
-			    "%s: no more (%d) PID space\n", Pn, sz);
-			Exit(1);
-		    }
-		}
 	    /*
-	     * Save the Solaris PGID and PID numbers in
-	     * local tables.
+	     * Save the PGID and PID numbers in local tables.
 	     */
 		if (Fpgid)
 		    Pgid[Np] = (int)pg.pid_id;
 		Pid[Np] = (int)pids.pid_id;
 	    /*
-	     * Save the proc structure in a local table.
+	     * If the proc structure came from kvm_getproc(), save it in the
+	     * local table.
 	     */
-		P[Np++] = *p;
+		if (!HasALLKMEM)
+		    P[Np] = *p;
+		Np++;
 	    }
 	/*
 	 * If not enough processes were saved in the local table, try again.
 	 */
 	    if (Np >= PROCMIN)
 		break;
-	    close_kvm();
+	    if (!HasALLKMEM)
+		close_kvm();
 	}
 /*
  * Quit if no proc structures were stored in the local table.
@@ -1238,7 +1529,7 @@ read_proc()
 	    (void) fprintf(stderr, "%s: can't read proc table\n", Pn);
 	    Exit(1);
 	}
-	if (Np < sz && !RptTm) {
+	if (Np < Npa && !RptTm) {
 
 	/*
 	 * Reduce the local proc structure table size to its minimum if
@@ -1267,6 +1558,7 @@ read_proc()
 		    "%s: can't reduce PID table to %d\n", Pn, Np);
 		Exit(1);
 	    }
+	    Npa = Np;
 	}
 }
 
