@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.22 2002/01/16 15:05:41 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.25 2004/03/10 23:53:41 abe Exp $";
 #endif
 
 
@@ -66,6 +66,30 @@ static char *rcsid = "$Id: dsock.c,v 1.22 2002/01/16 15:05:41 abe Exp $";
 
 #endif	/* !defined(HASIPv6) */
 
+#if	defined(HASTCPOPT)
+# if	solaris==20600
+#include <netinet/tcp.h>
+# endif	/* solaris==20600 */
+#include <netinet/tcp_timer.h>
+#include <netinet/tcp_var.h>
+
+# if	defined(TH_TIMER_NEEDED)
+#define	ACK_TIMER	TH_TIMER_NEEDED
+# else
+#  if	defined(TH_ACK_TIMER_NEEDED)
+#define	ACK_TIMER	TH_ACK_TIMER_NEEDED
+#  endif	/* defined(TH_ACK_TIMER_NEEDED) */
+# endif	/* defined(TH_TIMER_NEEDED */
+#endif	/* defined(HASTCPOPT) */
+
+#if	defined(HASSOOPT)
+# if	solaris<100000
+#define	KEEPALIVE_INTERVAL	tcp_keepalive_intrvl
+# else	/* solaris>=100000 */
+#define	KEEPALIVE_INTERVAL	tcp_ka_last_intrvl
+# endif	/* solaris<100000 */
+#endif	/* defined(HASSOOPT) */
+
 
 /*
  * print_tcptpi() - print TCP/TPI info
@@ -188,11 +212,11 @@ print_tcptpi(nl)
 	    ps++;
 	}
 
-# if	defined(HASTCPTPIQ)
+#if	defined(HASTCPTPIQ)
 	if (Ftcptpi & TCPTPI_QUEUES) {
 	    if (Lf->lts.rqs) {
 		if (Ffield)
-			putchar(LSOF_FID_TCPTPI);
+		    putchar(LSOF_FID_TCPTPI);
 		else {
 		    if (ps)
 			putchar(' ');
@@ -219,9 +243,196 @@ print_tcptpi(nl)
 		ps++;
 	    }
 	}
-# endif	/* defined(HASTCPTPIQ) */
+#endif	/* defined(HASTCPTPIQ) */
 
-# if	defined(HASTCPTPIW)
+#if	defined(HASSOOPT)
+	if (Ftcptpi & TCPTPI_FLAGS) {
+	    int opt;
+
+	    if ((opt = Lf->lts.opt)
+	    ||  Lf->lts.qlens || Lf->lts.qlims || Lf->lts.rbszs || Lf->lts.sbsz
+	    ) {
+		char sep = ' ';
+
+		if (Ffield)
+		    sep = LSOF_FID_TCPTPI;
+		else if (!ps)
+		    sep = '(';
+		(void) printf("%cSO", sep);
+		ps++;
+		sep = '=';
+
+# if	defined(SO_BROADCAST)
+		if (opt & SO_BROADCAST) {
+		    (void) printf("%cBROADCAST", sep);
+		    opt &= ~SO_BROADCAST;
+		    sep = ',';
+		}
+# endif	/* defined(SO_BROADCAST) */
+
+# if	defined(SO_DEBUG)
+		if (opt & SO_DEBUG) {
+		    (void) printf("%cDEBUG", sep);
+		    opt &= ~ SO_DEBUG;
+		    sep = ',';
+		}
+# endif	/* defined(SO_DEBUG) */
+
+# if	defined(SO_DGRAM_ERRIND)
+		if (opt & SO_DGRAM_ERRIND) {
+		    (void) printf("%cDGRAM_ERRIND", sep);
+		    opt &= ~SO_DGRAM_ERRIND;
+		    sep = ',';
+		}
+# endif	/* defined(SO_DGRAM_ERRIND) */
+
+# if	defined(SO_DONTROUTE)
+		if (opt & SO_DONTROUTE) {
+		    (void) printf("%cDONTROUTE", sep);
+		    opt &= ~SO_DONTROUTE;
+		    sep = ',';
+		}
+# endif	/* defined(SO_DONTROUTE) */
+
+# if	defined(SO_KEEPALIVE)
+		if (opt & SO_KEEPALIVE) {
+		    (void) printf("%cKEEPALIVE", sep);
+		    if (Lf->lts.kai)
+			(void) printf("=%d", Lf->lts.kai);
+		    opt &= ~SO_KEEPALIVE;
+		    sep = ',';
+		}
+# endif	/* defined(SO_KEEPALIVE) */
+
+# if	defined(SO_LINGER)
+		if (opt & SO_LINGER) {
+		    (void) printf("%cLINGER", sep);
+		    if (Lf->lts.ltm)
+			(void) printf("=%d", Lf->lts.ltm);
+		    opt &= ~SO_LINGER;
+		    sep = ',';
+		}
+# endif	/* defined(SO_LINGER) */
+
+# if	defined(SO_OOBINLINE)
+		if (opt & SO_OOBINLINE) {
+		    (void) printf("%cOOBINLINE", sep);
+		    opt &= ~SO_OOBINLINE;
+		    sep = ',';
+		}
+# endif	/* defined(SO_OOBINLINE) */
+
+		if (Lf->lts.qlens) {
+		    (void) printf("%cQLEN=%d", sep, Lf->lts.qlen);
+		    sep = ',';
+		}
+		if (Lf->lts.qlims) {
+		    (void) printf("%cQLIM=%lu", sep, Lf->lts.qlim);
+		    sep = ',';
+		}
+		if (Lf->lts.rbszs) {
+		    (void) printf("%cRCVBUF=%lu", sep, Lf->lts.rbsz);
+		    sep = ',';
+		}
+
+# if	defined(SO_REUSEADDR)
+		if (opt & SO_REUSEADDR) {
+		    (void) printf("%cREUSEADDR", sep);
+		    opt &= ~SO_REUSEADDR;
+		    sep = ',';
+		}
+# endif	/* defined(SO_REUSEADDR) */
+
+		if (Lf->lts.sbszs) {
+		    (void) printf("%cSNDBUF=%lu", sep, Lf->lts.sbsz);
+		    sep = ',';
+		}
+
+# if	defined(SO_USELOOPBACK)
+		if (opt & SO_USELOOPBACK) {
+		    (void) printf("%cUSELOOPBACK", sep);
+		    opt &= ~SO_USELOOPBACK;
+		    sep = ',';
+		}
+# endif	/* defined(SO_USELOOPBACK) */
+
+		if (opt)
+		    (void) printf("%cUNKNOWN=%#x", sep, opt);
+		if (Ffield)
+		    putchar(Terminator);
+	    }
+	}
+#endif	/* defined(HASSOOPT) */
+
+#if	defined(HASTCPOPT)
+	if (Ftcptpi & TCPTPI_FLAGS) {
+	    int topt;
+
+	    if ((topt = Lf->lts.topt) || Lf->lts.msss) {
+		char sep = ' ';
+
+		if (Ffield)
+		    sep = LSOF_FID_TCPTPI;
+		else if (!ps)
+		    sep = '(';
+		(void) printf("%cTF", sep);
+		ps++;
+		sep = '=';
+
+# if	defined(TF_ACKNOW)
+		if (topt & TF_ACKNOW) {
+		    (void) printf("%cACKNOW", sep);
+		    topt &= ~TF_ACKNOW;
+		    sep = ',';
+		}
+# endif	/* defined(TF_ACKNOW) */
+
+# if	defined(TF_DELACK)
+		if (topt & TF_DELACK) {
+		    (void) printf("%cDELACK", sep);
+		    topt &= ~TF_DELACK;
+		    sep = ',';
+		}
+# endif	/* defined(TF_DELACK) */
+
+		if (Lf->lts.msss) {
+		    (void) printf("%cMSS=%lu", sep, Lf->lts.mss);
+		    sep = ',';
+		}
+
+# if	defined(TF_NODELAY)
+		if (topt & TF_NODELAY) {
+		    (void) printf("%cNODELAY", sep);
+		    topt &= ~TF_NODELAY;
+		    sep = ',';
+		}
+# endif	/* defined(TF_NODELAY) */
+
+# if	defined(TF_NOOPT)
+		if (topt & TF_NOOPT) {
+		    (void) printf("%cNOOPT", sep);
+		    topt &= ~TF_NOOPT;
+		    sep = ',';
+		}
+# endif	/* defined(TF_NOOPT) */
+
+# if	defined(TF_SENTFIN)
+		if (topt & TF_SENTFIN) {
+		    (void) printf("%cSENTFIN", sep);
+		    topt &= ~TF_SENTFIN;
+		    sep = ',';
+		}
+# endif	/* defined(TF_SENTFIN) */
+
+		if (topt)
+		    (void) printf("%cUNKNOWN=%#x", sep, topt);
+		if (Ffield)
+		    putchar(Terminator);
+	    }
+	}
+#endif	/* defined(HASTCPOPT) */
+
+#if	defined(HASTCPTPIW)
 	if (Ftcptpi & TCPTPI_WINDOWS) {
 	    if (Lf->lts.rws) {
 		if (Ffield)
@@ -252,7 +463,7 @@ print_tcptpi(nl)
 		ps++;
 	    }
 	}
-# endif	/* defined(HASTCPTPIW) */
+#endif	/* defined(HASTCPTPIW) */
 
 	if (Ftcptpi && !Ffield && ps)
 	    putchar(')');
@@ -275,7 +486,17 @@ process_socket(sa, ty)
 	unsigned char *fa = (unsigned char *)NULL;
 	int fp = 0;
 	int i, lp;
+
+#if	solaris>=100000 && defined(HAS_IPCLASSIFIER_H)
+	struct conn_s ic;
+#define	ipc_v6laddr	conn_srcv6
+#define	ipc_v6faddr	conn_remv6
+#define	ipc_fport	conn_fport
+#define	ipc_lport	conn_lport
+#else	/* solaris<100000 || !defined(HAS_IPCLASSIFIER_H) */
 	struct ipc_s ic;
+#endif	/* solaris>=100000 && defined(HAS_IPCLASSIFIER_H) */
+
 	int ics = 0;
 	unsigned char *la = (unsigned char *)NULL;
 	struct module_info mi;
@@ -368,12 +589,13 @@ process_socket(sa, ty)
 	struct tcp_s tc;
 #endif	/* solaris<20600 */
 
-#if	solaris>=80000
+#if	solaris>=80000 && !defined(HAS_IPCLASSIFIER_H)
 	tcpb_t	tcb;
-#endif	/* solaris>=80000 */
+#endif	/* solaris>=80000 && !defined(HAS_IPCLASSIFIER_H) */
 
 	int tcs = 0;
 	tcph_t th;
+	int ths = 0;
 	struct ud_s {			/* should come from kernel source
 					 * file ../uts/common/inet/udp.c */
 	    uint udp_state;		/* TPI state */
@@ -450,9 +672,11 @@ process_socket(sa, ty)
 		continue;
 	    }
 	    if (strncasecmp(tbuf, "TCP", 3) == 0) {
+
+#if	solaris<=90000 || !defined(HAS_IPCLASSIFIER_H)
 		if (kread((KA_T)q.q_ptr, (char *)&tc, sizeof(tc)) == 0)
 
-#if	solaris>=80000
+# if	solaris>=80000
 		{
 		    if (tc.tcp_base
 		    &&  !kread((KA_T)tc.tcp_base, (char *)&tcb, sizeof(tcb)))
@@ -460,9 +684,17 @@ process_socket(sa, ty)
 		    tc.tcp_base = &tcb;		/* support for macroes */
 		    tcb.tcpb_tcp = &tc;		/* support for macroes */
 		}
-#else	/* solaris<80000 */
+# else	/* solaris<80000 */
 		    tcs = 1;
-#endif	/* solaris>=80000 */
+# endif	/* solaris>=80000 */
+#else	/* solaris>90000 && defined(HAS_IPCLASSIFIER_H) */
+		if (kread((KA_T)q.q_ptr, (char *)&ic, sizeof(ic)) == 0)
+		{
+		    if (ic.conn_tcp &&
+		    !kread((KA_T)ic.conn_tcp, (char *)&tc, sizeof(tc)))
+			ics = tcs = 1;
+		}
+#endif        /* solaris<=90000 || !defined(HAS_IPCLASSIFIER_H) */
 
 		continue;
 	    }
@@ -568,18 +800,24 @@ process_socket(sa, ty)
 #if	solaris!=20400 && solaris<80000
 		    la = (unsigned char *)&tc.tcp_iph.iph_src[0];
 #else	/* solaris==20400 || solaris<80000 */
-# if	solaris>=80000
-#  if	defined(HASIPv6)
+# if	solaris>=100000 && defined(HAS_IPCLASSIFIER_H)
+		    la = (af == AF_INET6) ? (unsigned char *)&ic.conn_srcv6
+		       :  (unsigned char *)IPv6_2_IPv4(&ic.conn_srcv6);
+# else	/* solaris<100000 || !defined(HAS_IPCLASSIFIER_H) */
+#  if	solaris>=80000
+#   if	defined(HASIPv6)
 		    la = (af == AF_INET6) ? (unsigned char *)&tcb.tcpb_ip_src_v6
 		       :  (unsigned char *)IPv6_2_IPv4(&tcb.tcpb_ip_src_v6);
-#  else	/* !defined(HASIPv6) */
+#   else	/* !defined(HASIPv6) */
 		    la = (unsigned char *)&tcb.tcpb_ip_src;
-#  endif	/* defined(HASIPv6) */
-# endif	/* solaris>=80000 */
+#   endif	/* defined(HASIPv6) */
+#  endif	/* solaris>=80000 */
+# endif	/* solaris>=100000 && defined(HAS_IPCLASSIFIER_H) */
 #endif	/* solaris!=20400 && !defined(HASIPv6) */
 
 		    if (tc.tcp_hdr_len && tc.tcp_tcph
 		    &&  !kread((KA_T)tc.tcp_tcph, (char *)&th, sizeof(th))) {
+			ths = 1;
 			s = (u_short *)&th.th_lport[0];
 			p = *s;
 		    }
@@ -640,6 +878,66 @@ process_socket(sa, ty)
 	     * Save TCP state information.
 	     */
 		if (tcs) {
+
+#if	defined(HASSOOPT)
+		    if (Ftcptpi & TCPTPI_FLAGS) {
+			if (tc.tcp_broadcast)
+			    Lf->lts.opt |= SO_BROADCAST;
+			if (tc.tcp_debug)
+			    Lf->lts.opt |= SO_DEBUG;
+			if (tc.tcp_dgram_errind)
+			    Lf->lts.opt |= SO_DGRAM_ERRIND;
+			if (tc.tcp_dontroute)
+			    Lf->lts.opt |= SO_DONTROUTE;
+			if (tc.KEEPALIVE_INTERVAL) {
+			    Lf->lts.opt |= SO_KEEPALIVE;
+			    Lf->lts.kai = (unsigned int)tc.KEEPALIVE_INTERVAL;
+			}
+			if (tc.tcp_linger) {
+			    Lf->lts.opt |= SO_LINGER;
+			    Lf->lts.ltm = (unsigned int)tc.tcp_lingertime;
+			}
+			if (tc.tcp_oobinline)
+			    Lf->lts.opt |= SO_OOBINLINE;
+			Lf->lts.qlen = (unsigned int)tc.tcp_conn_req_cnt_q;
+			Lf->lts.qlim = (unsigned int)tc.tcp_conn_req_max;
+			Lf->lts.qlens = Lf->lts.qlims = (unsigned char)1;
+
+# if	solaris>=80000
+#  if	defined(HAS_IPCLASSIFIER_H)
+			if (tc.tcp_reuseaddr)
+#  else	/* !defined(HAS_IPCLASSIFIER_H) */
+			if (tcb.tcpb_reuseaddr)
+#  endif	/* !defined(HAS_IPCLASSIFIER_H) */
+
+			    Lf->lts.opt |= SO_REUSEADDR;
+# endif	/* solaris>=80000 */
+
+			if (tc.tcp_useloopback)
+			    Lf->lts.opt |= SO_USELOOPBACK;
+#endif	/* defined(HASSOOPT) */
+
+#if	defined(HASTCPOPT)
+# if	defined(ACK_TIMER)
+			if (ths && (th.th_flags[0] & ACK_TIMER))
+			    Lf->lts.topt |= TF_DELACK;
+# endif	/* defined(ACK_TIMER) */
+
+# if	solaris<80000 || defined(HAS_IPCLASSIFIER_H)
+			Lf->lts.mss = (unsigned long)tc.tcp_mss;
+# else	/* solaris>=80000 && !defined(HAS_IPCLASSIFIER_H) */
+			Lf->lts.mss = (unsigned long)tcb.tcpb_mss;
+# endif	/* solaris<80000 || defined(HAS_IPCLASSIFIER_H) */
+
+			Lf->lts.msss = (unsigned char)1;
+
+			if (tc.tcp_naglim == 1L)
+			    Lf->lts.topt |= TF_NODELAY;
+			if (tc.tcp_fin_sent)
+			    Lf->lts.topt |= TF_SENTFIN;
+		    }
+#endif	/* defined(HASTCPOPT) */
+
 		    Lf->lts.type = 0;
 		    Lf->lts.state.i = (int)tc.tcp_state;
 		}
