@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dstore.c,v 1.15 2003/06/11 11:43:07 abe Exp $";
+static char *rcsid = "$Id: dstore.c,v 1.17 2004/03/10 23:53:41 abe Exp $";
 #endif
 
 
@@ -96,15 +96,20 @@ struct drive_Nl Drive_Nl[] = {
 
 	{ "nvops",	"nfs_vnodeops"		},
 	{ "n3vops",	"nfs3_vnodeops"		},
+	{ "n4vops",	"nfs4_vnodeops"		},
 	{ "nmvops",	"nm_vnodeops"		},
         { "nproc",	"nproc"			},
 	{ "pdvops",	"pcfs_dvnodeops"	},
 	{ "pfvops",	"pcfs_fvnodeops"	},
+	{ "pract",	"practive"		},
 	{ "prvops",	"prvnodeops"		},
-	{ "samvops",	"samfs_vnodeops"	},
+	{ "sam1vops",	"samfs_vnodeops"	},
+	{ "sam2vops",	"samfs_client_vnodeops"	},
 	{ "sgvops",	"segvn_ops"		},
 	{ "sckvops",	"sock_vnodeops"		},
 	{ "spvops",	"spec_vnodeops"		},
+	{ "sncavops",	"socknca_vnodeops"	},
+	{ "stpivops",	"socktpi_vnodeops"	},
 	{ "tvops",	"tmp_vnodeops"		},
 	{ "uvops",	"ufs_vnodeops"		},
 	{ "vvops",	"vx_vnodeops"		},
@@ -112,6 +117,7 @@ struct drive_Nl Drive_Nl[] = {
 	{ "vvfcops",	"fdd_chain_vnops"	},
 
 #if	solaris>=20500
+	{ "devops",	"dv_vnodeops"		},
 	{ "dvops",	"door_vnodeops"		},
 	{ "kbase",	"_kernelbase"		},
 #endif	/* solaris>=20500 */
@@ -126,6 +132,7 @@ struct drive_Nl Drive_Nl[] = {
 
 char **Fsinfo = NULL;			/* file system information */
 int Fsinfomax = 0;			/* maximum file system type */
+int HasALLKMEM = 0;			/* has ALLKMEM device */
 int HaveCloneMaj = 0;			/* clone major device number has
 					 * been identified and is in
 					 * CloneMaj */
