@@ -38,7 +38,7 @@
 # if	!defined(lint)
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: rnmh.c,v 1.8 2002/10/08 20:15:37 abe Exp $";
+static char *rcsid = "$Id: rnmh.c,v 1.10 2004/03/10 23:47:18 abe Exp $";
 # endif	/* !defined(lint) */
 
 #include "../lsof.h"
@@ -74,11 +74,14 @@ static char *rcsid = "$Id: rnmh.c,v 1.8 2002/10/08 20:15:37 abe Exp $";
  *	Define the names of these elements of struct NCACHE:
  *
  *		#define NCACHE_NM	<name>
- *		#define NCACHE_NMLEN	<name length
  *		#define NCACHE_NXT	<link to next entry>
  *		#define NCACHE_NODEADDR	<node address>
  *		#define NCACHE_PARADDR	<parent node address>
  *
+ *	Optionally define:
+ *
+ *		#define NCACHE_NMLEN	<name length
+ *
  *	Optionally define *both*:
  *
  *		#define	NCACHE_NODEID	<node capability ID>
@@ -104,6 +107,9 @@ static char *rcsid = "$Id: rnmh.c,v 1.8 2002/10/08 20:15:37 abe Exp $";
  * Note: if NCHNAMLEN is defined, the name is assumed to be in
  * NCACHE_NM[NCHNAMLEN]; if it isn't defined, the name is assumed to be in an
  * extension that begins at NCACHE_NM[0].
+ *
+ * Note: if NCACHE_NMLEN is not defined, then NCACHE_NM must be a pointer to
+ * a kernel allocated, NUL-terminated, string buffer.
  */
 
 
@@ -120,13 +126,17 @@ static char *rcsid = "$Id: rnmh.c,v 1.8 2002/10/08 20:15:37 abe Exp $";
  * Flags
  */
 
+# if	!defined(NCACHE_NMLEN)
+#undef	NCHNAMLEN
+# endif	/* !defined(NCACHE_NMLEN) */
+
 # if	!defined(NCACHE_VROOT)
 #define	NCACHE_VROOT	VROOT		/* vnode is root of its file system */
 # endif	/* !defined(NCACHE_VROOT) */
 
-#if	!defined(VNODE_VFLAG)
+# if	!defined(VNODE_VFLAG)
 #define	VNODE_VFLAG	v_flag
-#endif	/* !defined(VNODE_VFLAG) */
+# endif	/* !defined(VNODE_VFLAG) */
 
 
 /*
@@ -319,10 +329,20 @@ ncache_load()
 	char tbuf[32];
 	KA_T v;
 
-#if	!defined(NCHNAMLEN)
+# if	!defined(NCHNAMLEN)
 	int cin = sizeof(c.NCACHE_NM);
 	KA_T nmo = (KA_T)offsetof(struct NCACHE, NCACHE_NM);
-#endif	/* !defined(NCHNAMLEN) */
+# endif	/* !defined(NCHNAMLEN) */
+
+# if	!defined(NCACHE_NMLEN)
+	char nbf[MAXPATHLEN + 1];
+	int nbfl = (int)(sizeof(nbf) - 1);
+	KA_T nk;
+	char *np;
+	int rl;
+
+	nbf[nbfl] = '\0';
+# endif	/* !defined(NCACHE_NMLEN) */
 
 	if (!Fncache)
 	    return;
@@ -418,9 +438,54 @@ ncache_load()
 		if (kread(ka, (char *)&c, sizeof(c)))
 		    break;
 		knx = (KA_T)c.NCACHE_NXT;
-		if (!c.NCACHE_NODEADDR || !(len = c.NCACHE_NMLEN))
+		if (!c.NCACHE_NODEADDR)
 		    continue;
 
+# if	defined(NCACHE_NMLEN)
+		if (!(len = c.NCACHE_NMLEN))
+		    continue;
+# else	/* !defined(NCACHE_NMLEN) */
+	    /*
+	     * If it's possible to read the first three characters of the
+	     * do so and check for "." and "..".
+	     */
+		if (!c.NCACHE_NM
+		||  kread((KA_T)c.NCACHE_NM, nbf, 3))
+		    continue;
+		if (nbf[0] == '.') {
+		    if (!nbf[1]
+		    ||  ((nbf[1] == '.') && !nbf[2]))
+			continue;
+		}
+	   /*
+	    * Read the rest of the name, 32 characters at a time, until a NUL
+	    * character has been read or nbfl characters have been read.
+	    */
+		nbf[3] = '\0';
+		if ((len = (int)strlen(nbf)) < 3) {
+		    if (!len)
+			continue;
+		} else {
+		    for (np = &nbf[3]; len < nbfl; np += rl) {
+			if ((rl = nbfl - len) > 32) {
+			    rl = 32;
+			    nbf[len + rl] = '\0';
+			}
+			nk = (KA_T)((char *)c.NCACHE_NM + len);
+			if (kread(nk, np, rl)) {
+			    rl = -1;
+			    break;
+			}
+			rl = (int)strlen(np);
+			len += rl;
+			if (rl < 32)
+			    break;
+		    }
+		    if (rl < 0)
+			continue;
+		}
+# endif	/* defined(NCACHE_NMLEN) */
+
 	    /*
 	     * Allocate a cache entry long enough to contain the name and
 	     * move the name to it.
@@ -455,6 +520,7 @@ ncache_load()
 # if	defined(NCHNAMLEN)
 		(void) strncpy(lc->nm, c.NCACHE_NM, len);
 # else	/* !defined(NCHNAMLEN) */
+#  if	defined(NCACHE_NMLEN)
 		if ((len < 3) && (cin > 1)) {
 
 		/*
@@ -485,8 +551,11 @@ ncache_load()
 		    }
 		} else
 		    (void) strncpy(lc->nm, c.NCACHE_NM, len);
-# endif	/* defined(NCHNAMLEN) */
+#  else	/* !defined(NCACHE_NMLEN) */
+		(void) strncpy(lc->nm, nbf, len);
+#  endif	/* defined(NCACHE_NMLEN) */
 
+# endif	/* defined(NCHNAMLEN) */
 		lc->nm[len] = '\0';
 	    /*
 	     * Complete the new local cache entry and link it to the previous
