@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: lsof.h,v 1.52 2004/03/10 23:42:12 abe Exp $
+ * $Id: lsof.h,v 1.53 2004/07/06 19:09:10 abe Exp $
  */
 
 
@@ -188,7 +188,11 @@ struct l_dev {
 #undef	EMPTY
 # endif	/* defined(EMPTY) */
 
+# if	defined(HASUTMPX)
+#include <utmpx.h>
+# else	/* !defined(HASUTMPX) */
 #include <utmp.h>
+# endif	/* defined(HASUTMPX) */
 
 extern int errno;
 extern char *optarg;
@@ -242,11 +246,15 @@ extern int optind;
 					 * addresses in 0x... format */
 # endif	/* !defined(KA_T_FMT_X) */
 
-#if	!defined(LOGINML)
+# if	!defined(LOGINML)
+#  if	defined(HASUTMPX)
+static struct utmpx dummy_utmp;		/* to get login name length */
+# else	/* !defined(HASUTMPX) */
 static struct utmp dummy_utmp;		/* to get login name length */
+# endif	/* defined(HASUTMPX) */
 #define	LOGINML		sizeof(dummy_utmp.ut_name)
 					/* login name length */
-#endif	/* !defined(LOGINML) */
+# endif	/* !defined(LOGINML) */
 
 #define	LPROCINCR	128		/* Lproc[] allocation increment */
 #define	LSOF_URL	"ftp://lsof.itap.purdue.edu/pub/tools/unix/lsof/"
@@ -519,6 +527,7 @@ extern char **Dstk;
 extern int Dstkn;
 extern int Dstkx;
 extern int ErrStat;
+extern uid_t Euid;
 extern int Fand;
 extern int Fblock;
 extern int Ffield;
@@ -672,6 +681,10 @@ struct lfile {
 
 # if	defined(HASSOSTATE)
 	    unsigned int ss;		/* socket state */
+#  if	defined(HASSBSTATE)
+	    unsigned int sbs_rcv;	/* receive socket buffer state */
+	    unsigned int sbs_snd;	/* send socket buffer state */
+#  endif	/* defined(HASSBSTATE) */
 # endif	/* defined(HASSOSTATE) */
 
 # if	defined(HASTCPOPT)
@@ -743,6 +756,8 @@ struct lproc {
 extern struct lproc *Lp, *Lproc;
 
 extern char *Memory;
+extern int MntSup;
+extern char *MntSupP;
 
 # if	defined(HASPROCFS)
 extern struct mounts *Mtprocfs;
