@@ -34,7 +34,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: main.c,v 1.45 2004/03/10 23:43:40 abe Exp $";
+static char *rcsid = "$Id: main.c,v 1.46 2004/07/06 19:09:32 abe Exp $";
 #endif
 
 
@@ -126,7 +126,8 @@ main(argc, argv)
 	Mypid = getpid();
 	if ((Mygid = (gid_t)getgid()) != getegid())
 	    Setgid = 1;
-	if ((Myuid = (uid_t)getuid()) && !geteuid())
+	Euid = geteuid();
+	if ((Myuid = (uid_t)getuid()) && !Euid)
 	    Setuidroot = 1;
 	if (!(Namech = (char *)malloc(MAXPATHLEN + 1))) {
 	    (void) fprintf(stderr, "%s: no space for name buffer\n", Pn);
@@ -157,11 +158,11 @@ main(argc, argv)
 	    "",
 #endif	/* defined(HASKOPT) */
 
-#if	defined(HASMOPT)
+#if	defined(HASMOPT) || defined(HASMNTSUP)
 	    "m:",
-#else	/* !defined(HASMOPT) */
+#else	/* !defined(HASMOPT) && !defined(HASMNTSUP) */
 	    "",
-#endif	/* defined(HASMOPT) */
+#endif	/* defined(HASMOPT) || defined(HASMNTSUP) */
 
 #if	defined(HASPPID)
 	    "R",
@@ -204,6 +205,10 @@ main(argc, argv)
 		if (!GOv || *GOv == '-' || *GOv == '+') {
 		    (void) fprintf(stderr, "%s: -A not followed by path\n", Pn);
 		    err = 1;
+		    if (GOv) {
+			GOx1 = GObk[0];
+			GOx2 = GObk[1];
+		    }
 		} else
 		    AFSApath = GOv;
 		break;
@@ -220,6 +225,10 @@ main(argc, argv)
 			(void) fprintf(stderr,
 			    "%s: +c not followed by width number\n", Pn);
 			err = 1;
+			if (GOv) {
+			    GOx1 = GObk[0];
+			    GOx2 = GObk[1];
+			}
 		    } else
 			CmdLim = atoi(GOv);
 		    break;
@@ -448,6 +457,10 @@ main(argc, argv)
 		if (!GOv || *GOv == '-' || *GOv == '+') {
 		    (void) fprintf(stderr, "%s: -k not followed by path\n", Pn);
 		    err = 1;
+		    if (GOv) {
+			GOx1 = GObk[0];
+			GOx2 = GObk[1];
+		    }
 		} else
 		    Nmlst = GOv;
 		break;
@@ -489,15 +502,50 @@ main(argc, argv)
 		}
 		break;
 
-#if	defined(HASMOPT)
+#if	defined(HASMOPT) || defined(HASMNTSUP)
 	    case 'm':
-		if (!GOv || *GOv == '-' || *GOv == '+') {
-		    (void) fprintf(stderr, "%s: -m not followed by path\n", Pn);
+		if (GOp == '-') {
+
+# if	defined(HASMOPT)
+		    if (!GOv || *GOv == '-' || *GOv == '+') {
+			(void) fprintf(stderr,
+			    "%s: -m not followed by path\n", Pn);
+			err = 1;
+			if (GOv) {
+			    GOx1 = GObk[0];
+			    GOx2 = GObk[1];
+			}
+		    } else
+			Memory = GOv;
+# else	/* !defined(HASMOPT) */
+		    (void) fprintf(stderr, "%s: -m not supported\n", Pn);
 		    err = 1;
-		} else
-		    Memory = GOv;
+# endif	/* defined(HASMOPT) */
+
+		} else if (GOp == '+') {
+
+# if	defined(HASMNTSUP)
+		    if (!GOv || *GOv == '-' || *GOv == '+') {
+			MntSup = 1;
+			if (GOv) {
+			    GOx1 = GObk[0];
+			    GOx2 = GObk[1];
+			}
+		    } else {
+			MntSup = 2;
+			MntSupP = GOv;
+		    }
+# else	/* !defined(HASMNTSUP) */
+		    (void) fprintf(stderr, "%s: +m not supported\n", Pn);
+		    err = 1;
+# endif	/* defined(HASMNTSUP) */
+
+		} else {
+		    (void) fprintf(stderr, "%s: %cm not supported\n", GOp, Pn);
+		    err = 1;
+		}
 		break;
-#endif	/* defined(HASMOPT) */
+#endif	/* defined(HASMOPT) || defined(HASMNTSUP) */
 
 	    case 'M':
 		FportMap = (GOp == '+') ? 1 : 0;
@@ -848,6 +896,17 @@ main(argc, argv)
 	SzOffFmt_dv = sv_fmt_str(options);
 	(void) snpf(options, sizeof(options), "%%#%sx", SZOFFPSPEC);
 	SzOffFmt_x = sv_fmt_str(options);
+
+#if	defined(HASMNTSUP)
+/*
+ * Report mount supplement information, as requested.
+ */
+	if (MntSup == 1) {
+	    (void) readmnt();
+	    Exit(0);
+	}
+#endif	/* defined(HASMNTSUP) */
+
 /*
  * Gather and report process information every RptTm seconds.
  */
