@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: proto.h,v 1.30 2002/12/05 11:58:44 abe Exp $
+ * $Id: proto.h,v 1.31 2003/03/21 17:26:14 abe Exp $
  */
 
 
@@ -185,6 +185,10 @@ _PROTOTYPE(extern char *print_fflags,(long ffg, long pof));
 _PROTOTYPE(extern int readgnode,(KA_T ga, struct gnode *g));
 # endif	/* defined(HASGNODE) */
 
+# if	defined(HASKQUEUE)
+_PROTOTYPE(extern void process_kqueue,(KA_T ka));
+# endif	/* defined(HASKQUEUE) */
+
 # if	defined(HASHSNODE)
 _PROTOTYPE(extern int readhsnode,(KA_T ha, struct hsnode *h));
 # endif	/* defined(HASHSNODE) */
